/*
 * Revision 1.4	 2004/07/13  21:40:32  blurcode
 * Spawn monster code added
 * Common teleport locations for admin added
 * Ability to teleport characters added
 * Logout character code added
 * Extra exception catchers added 
 * 
 * Revision 1.3  2004/07/11 23:39:52  whatev
 * Teleport added to admininterface
 * 
 * Revision 1.2  2004/07/09 22:33:00  blurcode
 * Search for character added
 * List characters per page code added
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

package net.sf.l2j.gameserver;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.logging.Logger;
import java.lang.Integer;

import net.sf.l2j.gameserver.CharTemplateTable;
import net.sf.l2j.gameserver.Announcements;
import net.sf.l2j.gameserver.model.L2BuyList;
import net.sf.l2j.gameserver.model.L2ItemInstance;
import net.sf.l2j.gameserver.model.L2MonsterInstance;
import net.sf.l2j.gameserver.model.L2NpcData;
import net.sf.l2j.gameserver.model.L2PcInstance;
import net.sf.l2j.gameserver.model.L2Spawn;
import net.sf.l2j.gameserver.model.L2World;
import net.sf.l2j.gameserver.model.L2Skill;
import net.sf.l2j.gameserver.serverpackets.ActionFailed;
import net.sf.l2j.gameserver.serverpackets.BuyList;
import net.sf.l2j.gameserver.serverpackets.CreatureSay;
import net.sf.l2j.gameserver.serverpackets.ItemList;
import net.sf.l2j.gameserver.serverpackets.LeaveWorld;
import net.sf.l2j.gameserver.serverpackets.MagicSkillUser;
import net.sf.l2j.gameserver.serverpackets.NpcHtmlMessage;
import net.sf.l2j.gameserver.serverpackets.SystemMessage;
import net.sf.l2j.gameserver.serverpackets.TeleportToLocation;
import net.sf.l2j.gameserver.templates.L2Npc;

public class AdminCommands
{
	private static Logger _log = Logger.getLogger(AdminCommands.class.getName());
	private static AdminCommands _instance;
	
	public static AdminCommands getInstance()
	{
		if (_instance == null)
		{
			_instance = new AdminCommands();
		}
		
		return _instance;
	}
	
	private static String _characterToManipulate;
	
	public void handleCommands(ClientThread client, String command)
	{
		L2PcInstance activeChar = client.getActiveChar();
		
		if (command.equals("admin_show"))
		{
			showMainPage(client);
		}
		else if (command.equals("admin_list_announcements"))
		{
			Announcements.getInstance().listAnnouncements(activeChar);
		}
		else if (command.equals("admin_reload_announcements"))
		{
			Announcements.getInstance().loadAnnouncements();
			Announcements.getInstance().listAnnouncements(activeChar);
		}
		else if (command.equals("admin_announce_announcements"))
		{
			L2PcInstance[] players = L2World.getInstance().getAllPlayers();
			for (int i = 0; i < players.length; i++)
			{
				Announcements.getInstance().showAnnouncements(players[i]);
			}
			Announcements.getInstance().listAnnouncements(activeChar);
		}
		else if (command.startsWith("admin_add_announcement"))
		{
			//FIXME the player can send only 16 chars (if you try to send more it sends null), remove this function or not?
			if (!command.equals("admin_add_announcement"))
			{
				String val = command.substring(23);
				Announcements.getInstance().addAnnouncement(val);
				Announcements.getInstance().listAnnouncements(activeChar);
			}
		}
		else if (command.startsWith("admin_del_announcement"))
		{
			int val = new Integer(command.substring(23)).intValue();
			Announcements.getInstance().delAnnouncement(val);
			Announcements.getInstance().listAnnouncements(activeChar);
		}
		else if (command.equals("admin_show_moves"))
		{
			showHelpPage(client, "teleports.htm");
		}
		else if (command.equals("admin_show_spawns"))
		{
			showHelpPage(client, "spawns.htm");
		}
		if (command.startsWith("admin_buy"))
		{
			handleBuyRequest(client, command.substring(10));
		}
		else if (command.startsWith("admin_spawn_index"))
		{
			String val = command.substring(18);
			showHelpPage(client, "spawns/" + val + ".htm");
		}
		else if (command.equals("admin_character_disconnect"))
		{ 
			disconnectCharacter(client);			
		}		
		else if (command.equals("admin_show_teleport"))
		{
			showTeleportCharWindow(client);
		}
		else if (command.equals("admin_teleport_to_character"))
		{
			teleportToCharacter(client);
		}
		else if (command.startsWith("admin_move_to"))
		{
			try
			{
				String val = command.substring(14);
				teleportTo(client, val);
			}
			catch (StringIndexOutOfBoundsException e)
			{	//Case of empty co-ordinates
				SystemMessage sm = new SystemMessage(614);
				sm.addString("Wrong or no Co-ordinates given.");
				activeChar.sendPacket(sm);
				
				showMainPage(client); //back to admin window
			}		
		}
		else if (command.startsWith("admin_help"))
		{
			try
			{
				String val = command.substring(11);
				showHelpPage(client, val);
			}
			catch (StringIndexOutOfBoundsException e)
			{
				//case of empty filename
			}
			
		}
		else if (command.startsWith("admin_character_list"))
		{
			try
			{
				String val = command.substring(21); 
				showCharacterList(client, val);
			}
			catch (StringIndexOutOfBoundsException e)
			{
				//Case of empty character name
			}
			
		}
		else if (command.startsWith("admin_show_characters"))
		{
			try
			{   
				String val = command.substring(22);
				int page = Integer.parseInt(val);
				listCharacters(client, page);
			}
			catch (StringIndexOutOfBoundsException e)
			{
				//Case of empty page
			}
		}
		else if (command.startsWith("admin_find_character"))
		{
			try
			{
				String val = command.substring(21); 
				findCharacter(client, val);
			}
			catch (StringIndexOutOfBoundsException e)
			{	//Case of empty character name
				SystemMessage sm = new SystemMessage(614);
				sm.addString("You didnt enter a character name to find.");
				activeChar.sendPacket(sm);
				
				listCharacters(client, 0);
			}			
		}
		else if (command.startsWith("admin_teleport_character"))
		{
			try
			{
				String val = command.substring(25); 
				teleportCharacter(client, val);
			}
			catch (StringIndexOutOfBoundsException e)
			{
				//Case of empty co-ordinates
				SystemMessage sm = new SystemMessage(614);
				sm.addString("Wrong or no Co-ordinates given.");
				activeChar.sendPacket(sm);
				
				showTeleportCharWindow(client); //back to character teleport
			}
		} 		
		else if (command.startsWith("admin_spawn_monster"))
		{
			try
			{
				String val = command.substring(20);
				spawnMonster(client, val);
			}
			catch  (StringIndexOutOfBoundsException e)
			{
				//Case of wrong monster data
			}
		}
		else if (command.startsWith("admin_server_shutdown"))
		{	
			try
			{
				int val = Integer.parseInt(command.substring(22)); 
				serverShutdown(client, val);
			}
			catch (StringIndexOutOfBoundsException e)
			{
				SystemMessage sm = new SystemMessage(614);
				sm.addString("You didnt enter the seconds untill server shutdowns.");
				activeChar.sendPacket(sm);
				
				serverShutdown(client, 0);
			}
		}
		else if (command.equals("admin_gm_shops"))
		{
			showHelpPage(client, "gmshops.htm");
		}
		else if (command.equals("admin_itemcreate"))
		{
			showHelpPage(client, "itemcreation.htm");
		}		
		else if (command.startsWith("admin_create_item"))
		{
			try
			{
				String val = command.substring(18);
				createItem(client, val);
			}
			
			catch (StringIndexOutOfBoundsException e)
			{
				SystemMessage sm = new SystemMessage(614);
				sm.addString("You didnt enter an Item ID to Create.");
				activeChar.sendPacket(sm);
			}
		}
		
	}
	
	/**
	 * @param client
	 */
	private void handleBuyRequest(ClientThread client, String command)
	{
		L2PcInstance player = client.getActiveChar();
		
		int val = -1;
		
		try
		{
			val = Integer.parseInt(command);
		}
		catch (Exception e)
		{
			_log.warning("admin buylist failed:"+command);
		}
		
		L2BuyList list = TradeController.getInstance().getBuyList(val);
		
		if (list != null)
		{	
			BuyList bl  = new BuyList(list, player.getAdena());
			player.sendPacket(bl);
		}
		else
		{
			_log.warning("no buylist with id:" +val);
		}
		
		player.sendPacket( new ActionFailed() );
	}

	public void showMainPage(ClientThread client)
	{
		L2PcInstance activeChar = client.getActiveChar();
		L2PcInstance[] players = L2World.getInstance().getAllPlayers();		
		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);
		
		StringBuffer replyMSG = new StringBuffer("<html><title>Server Status</title>");
		replyMSG.append("<body>");
		replyMSG.append("<table>");		
		replyMSG.append("<tr><td>Players Online: " + players.length + "</td></tr>");				
		replyMSG.append("<tr><td>Server Software: This Server is running L2J version 0.2 created by L2Chef and the L2J team.</td></tr>");
		replyMSG.append("<tr><td>Used Memory: " + (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()));
		replyMSG.append(" bytes</td></tr>");
		replyMSG.append("</table>");		
		replyMSG.append("<br>");
		
		replyMSG.append("<br>");
		replyMSG.append("<center><button value=\"Help\" action=\"bypass -h admin_help admhelp.htm\" width=110 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("<center><button value=\"Shutdown Server\" action=\"bypass -h admin_server_shutdown 0\" width=110 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");	
		replyMSG.append("<center><button value=\"List Characters\" action=\"bypass -h admin_show_characters 0\" width=110 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("<center><button value=\"Move to Location\" action=\"bypass -h admin_show_moves\" width=110 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("<center><button value=\"Spawn a Monster\" action=\"bypass -h admin_show_spawns\" width=110 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("<center><button value=\"Item Creation\" action=\"bypass -h admin_itemcreate\" width=110 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("<center><button value=\"GM Shops\" action=\"bypass -h admin_gm_shops\" width=110 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("<center><button value=\"Announcements\" action=\"bypass -h admin_list_announcements\" width=110 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("<br><br>");
		replyMSG.append("<right><button value=\"Close\" action=\"bypass -h admin_close_window\" width=100 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></right>"); //This does nothing
		replyMSG.append("</body></html>");
		
		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);	
	}
	
	
	private void teleportTo(ClientThread client, String Cords)
	{
		L2PcInstance activeChar = client.getActiveChar();
		
		StringTokenizer st = new StringTokenizer(Cords);
		String x1 = st.nextToken();
		int x = Integer.parseInt(x1);
		String y1 = st.nextToken();
		int y = Integer.parseInt(y1);
		String z1 = st.nextToken();
		int z = Integer.parseInt(z1);
				
		TeleportToLocation tele = new TeleportToLocation(activeChar, x,y,z);
		activeChar.sendPacket(tele);
		activeChar.setX(x);
		activeChar.setY(y);
		activeChar.setZ(z);
			
		SystemMessage sm = new SystemMessage(614);
		sm.addString("You have been teleported to " + Cords);
		activeChar.sendPacket(sm);
		
		showMainPage(client); //Back to start
	}

	private void createItem(ClientThread client, String itemnum)
	{
		L2PcInstance activeChar = client.getActiveChar();
		
		int item = Integer.parseInt(itemnum, 10);
		
		L2ItemInstance createditem = ItemTable.getInstance().createItem(item);
		activeChar.getInventory().addItem(createditem);		

		ItemList il = new ItemList(activeChar, true);
		activeChar.sendPacket(il);
		
		SystemMessage sm = new SystemMessage(614);
		sm.addString("You have spawned item number " + item + " in your inventory");
		activeChar.sendPacket(sm);
				
		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);		
		StringBuffer replyMSG = new StringBuffer("<html><title>Item Creation Complete(I Hope)</title>");
		replyMSG.append("<body>");
		replyMSG.append("<center><button value=\"Back\" action=\"bypass -h admin_show\" width=40 height=15></center>");
		replyMSG.append("</body></html>");
		
		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
	}
	
	private void listCharacters(ClientThread client, int page)
	{	
		L2PcInstance activeChar = client.getActiveChar();
		L2PcInstance[] players = L2World.getInstance().getAllPlayers();

		int MaxCharactersPerPage = 20;
		int MaxPages = players.length / MaxCharactersPerPage;
		int modulus = players.length % MaxCharactersPerPage;
			
		if (modulus!=0)
		{
			MaxPages = MaxPages+1;
		}
		
		//Check if number of users changed
		if (page>MaxPages)
		{
			page=MaxPages;
		}

		int CharactersStart = MaxCharactersPerPage*page;
		int CharactersEnd = players.length-CharactersStart;		
		
		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);		

		StringBuffer replyMSG = new StringBuffer("<html><title>Characters List</title>");
		replyMSG.append("<body>");
		
		for (int x=0; x<MaxPages; x++)
		{
			replyMSG.append("<a action=\"bypass -h admin_show_characters " + x + "\">Page" + x+1 + "</a>\t");	
		}			
		replyMSG.append("<br>");

		//List Players in a Table
		replyMSG.append("<table>");		
		replyMSG.append("<tr><td>Name</td><td>Class</td><td>Level</td></tr>");
		for (int i = CharactersStart; i < CharactersEnd; i++)
		{	//Add player info into new Table row
			replyMSG.append("<tr><td>" + "<a action=\"bypass -h admin_character_list " + players[i].getName() + "\">" + players[i].getName() + "</a>" + "</td><td>" + CharTemplateTable.getInstance().getTemplate(players[i].getClassId()).getClassName() + "</td><td>" + players[i].getLevel() + "</td></tr>");
		}
		replyMSG.append("</table>");
		replyMSG.append("---------<p>");
		replyMSG.append("You can find a character by writing his name<p> and clicking Find bellow:");
		replyMSG.append("<edit var=\"character_name\" width=110></center>");
		replyMSG.append("<button value=\"Find\" action=\"bypass -h admin_find_character $character_name\" width=40 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\">");
		replyMSG.append("<center><button value=\"Back\" action=\"bypass -h admin_show\" width=40 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("</body></html>");
		
		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);			
	}
	
	private void showCharacterList(ClientThread client, String CharName)
	{
		L2PcInstance activeChar = client.getActiveChar();	
		L2PcInstance player = L2World.getInstance().getPlayer(CharName);		
		NpcHtmlMessage adminReply = new NpcHtmlMessage(5); 
		
		StringBuffer replyMSG = new StringBuffer("<html><title>Character Information</title>");
		replyMSG.append("<body>");
		
		replyMSG.append("<br>");
		replyMSG.append("<center>" + player.getName() + "</center><p>");	
		replyMSG.append("Clan: " + ClanTable.getInstance().getClan(player.getClanId()) + "<p>");		
		replyMSG.append("Lv: " + player.getLevel() + " " + CharTemplateTable.getInstance().getTemplate(player.getClassId()).getClassName() + "<p>");
		replyMSG.append("Exp: " + player.getExp() + "<p>");				


		replyMSG.append("<table>");		
		replyMSG.append("<tr><td>HP</td><td>" + player.getCurrentHp() + " / " + player.getMaxHp() + "</td><td>MP</td><td>" + player.getCurrentMp() + " / " + player.getMaxMp() + "</td></tr>");
		replyMSG.append("<tr><td>SP</td><td>" + player.getSp() + "</td><td>Load</td><td>" + player.getCurrentLoad() + "</td></tr>");		
		replyMSG.append("</table>");
		replyMSG.append("<br>");
		
		//Combat
		replyMSG.append("<table>");		
		replyMSG.append("<tr><td>Combat</td></tr>");
		replyMSG.append("</table>");

		replyMSG.append("<table>");		
		replyMSG.append("<tr><td>P.ATK</td><td>" + player.getPhysicalAttack() + "</td><td>M.ATK</td><td>" + player.getMagicalAttack() + "</td></tr>");
		replyMSG.append("<tr><td>P.DEF</td><td>" + player.getPhysicalDefense() + "</td><td>M.DEF</td><td>" + player.getMagicalDefense() + "</td></tr>");
		replyMSG.append("<tr><td>Accuracy</td><td>" + player.getAccuracy() + "</td><td>Evasion</td><td>" + player.getEvasionRate() + "</td></tr>");
		replyMSG.append("<tr><td>Critical</td><td>" + player.getCriticalHit() + "</td><td>Speed</td><td>" + player.getRunSpeed() + "</td></tr>");
		replyMSG.append("<tr><td>ATK Spd</td><td>" + player.getPhysicalSpeed() + "</td><td>Casting Spd</td><td>" + player.getMagicalSpeed() + "</td></tr>");		
		replyMSG.append("</table>");
		replyMSG.append("<br>");
		//Basic
		replyMSG.append("<table>");		
		replyMSG.append("<tr><td>Basic</td></tr>");
		replyMSG.append("</table>");

		replyMSG.append("<table>");		
		replyMSG.append("<tr><td>STR</td><td>" + player.getStr() + "</td><td>DEX</td><td>" + player.getDex() + "</td><td>CON</td><td>" + player.getCon() + "</td></tr>");
		replyMSG.append("<tr><td>INT</td><td>" + player.getInt() + "</td><td>WIT</td><td>" + player.getWit() + "</td><td>MEN</td><td>" + player.getMen() + "</td></tr>");		
		replyMSG.append("</table>");
		replyMSG.append("<br>");
		//Social
		replyMSG.append("<table>");		
		replyMSG.append("<tr><td>Social</td></tr>");
		replyMSG.append("</table>");
		
		replyMSG.append("<table>");		
		replyMSG.append("<tr><td>Karma</td><td>" + player.getKarma() + "</td><td>PvP</td><td>" + player.getPvpFlag() + " / " + player.getPvpKills() +  "</td></tr>");		
		replyMSG.append("</table><p>");				

		replyMSG.append("Character Co-ordinates: " + player.getX() + " " + player.getY() + " " + player.getZ());
		replyMSG.append("<br>");
		replyMSG.append("<center><button value=\"Logout Character\" action=\"bypass -h admin_character_disconnect\" width=130 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("<center><button value=\"Teleport Character\" action=\"bypass -h admin_show_teleport\" width=130 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("<center><button value=\"Teleport to Character\" action=\"bypass -h admin_teleport_to_character\" width=130 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("<br><center><button value=\"Back\" action=\"bypass -h admin_show\" width=40 height=15></center>");
		replyMSG.append("</body></html>");

		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);
		
		_characterToManipulate = player.getName();
	}
	
	private void disconnectCharacter(ClientThread client)
	{
		L2PcInstance activeChar = client.getActiveChar();	
		L2PcInstance player = L2World.getInstance().getPlayer(_characterToManipulate);		
		
		if (player.getName().equals(activeChar.getName()))
		{		
			SystemMessage sm = new SystemMessage(614);
			sm.addString("You cannot logout your character.");
			activeChar.sendPacket(sm);
		}
		else
		{				
			SystemMessage sm = new SystemMessage(614);
			sm.addString("Character " + player.getName() + " disconnected from server.");
			activeChar.sendPacket(sm);
			
			//Logout Character
			LeaveWorld ql = new LeaveWorld();
			player.sendPacket(ql);
			
			try
			{
				player.getNetConnection().close();
			}
			catch (IOException e)
			{
				// just to make sure we try to kill the connection 
			}
		}
		
		showMainPage(client); //Back to start
	}
	
	private void findCharacter(ClientThread client, String CharacterToFind)
	{
		L2PcInstance activeChar = client.getActiveChar();
		L2PcInstance[] players = L2World.getInstance().getAllPlayers();
		NpcHtmlMessage adminReply = new NpcHtmlMessage(5);
		int CharactersFound = 0;
		
		StringBuffer replyMSG = new StringBuffer("<html><title>Character Search</title>");		
		replyMSG.append("<body>");
		replyMSG.append("<br>");
		
		replyMSG.append("<table>");		
		replyMSG.append("<tr><td>Name</td><td>Class</td><td>Level</td></tr>");
		
		for (int i = 0; i < players.length; i++)
		{	//Add player info into new Table row
			
			if (players[i].getName().startsWith((CharacterToFind)))
			{
				CharactersFound = CharactersFound+1;
				replyMSG.append("<tr><td>" + "<a action=\"bypass -h admin_character_list " + players[i].getName() + "\">" + players[i].getName() + "</a>" + "</td><td>" + CharTemplateTable.getInstance().getTemplate(players[i].getClassId()).getClassName() + "</td><td>" + players[i].getLevel() + "</td></tr>");
			}	
		}
		replyMSG.append("</table>");
		
		if (CharactersFound==0)
		{
			replyMSG.append("<br>Your search did not find any characters. Please try again:");
			replyMSG.append("<edit var=\"character_name\" width=110>");
			replyMSG.append("<button value=\"Find\" action=\"bypass -h admin_find_character $character_name\" width=40 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\">");
			replyMSG.append("<br>");
		}
		else
		{
			replyMSG.append("<br>Found " + CharactersFound + " character");
			
			if (CharactersFound==1)
			{
				replyMSG.append(".");
			}
			else 
			{
				if (CharactersFound>1)
				{
					replyMSG.append("s.");
				}
			}

		}
		
		replyMSG.append("<br>");
		replyMSG.append("<center><button value=\"Back\" action=\"bypass -h admin_show\" width=40 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");		
		replyMSG.append("</body></html>");
		
		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);			
	}
	
	private void showTeleportCharWindow(ClientThread client)
	{
		L2PcInstance activeChar = client.getActiveChar();	
		L2PcInstance player = L2World.getInstance().getPlayer(_characterToManipulate);		
		NpcHtmlMessage adminReply = new NpcHtmlMessage(5); 
		
		StringBuffer replyMSG = new StringBuffer("<html><title>Teleport Character</title>");
		replyMSG.append("<body>");
		replyMSG.append("The character you will teleport is " + player.getName() + ".");
		replyMSG.append("<br>");
		
		replyMSG.append("Co-ordinate x");
		replyMSG.append("<edit var=\"char_cord_x\" width=110>");
		replyMSG.append("Co-ordinate y");
		replyMSG.append("<edit var=\"char_cord_y\" width=110>");
		replyMSG.append("Co-ordinate z");
		replyMSG.append("<edit var=\"char_cord_z\" width=110>");
		replyMSG.append("<button value=\"Teleport\" action=\"bypass -h admin_teleport_character $char_cord_x $char_cord_y $char_cord_z\" width=60 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\">");	
		replyMSG.append("<button value=\"Teleport near you\" action=\"bypass -h admin_teleport_character " + activeChar.getX() + " " + activeChar.getY() + " " + activeChar.getZ() + "\" width=115 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\">");
		replyMSG.append("<center><button value=\"Back\" action=\"bypass -h admin_show\" width=40 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\"></center>");
		replyMSG.append("</body></html>");
		
		adminReply.setHtml(replyMSG.toString());
		activeChar.sendPacket(adminReply);			
	}
	
	private void teleportCharacter(ClientThread client, String Cords)
	{
		L2PcInstance activeChar = client.getActiveChar();
		L2PcInstance player = L2World.getInstance().getPlayer(_characterToManipulate);
		
		if (player.getName().equals(activeChar.getName()))
		{
			SystemMessage sm = new SystemMessage(614);
			sm.addString("You cannot teleport your character.");
			player.sendPacket(sm);
		}
		else
		{
			StringTokenizer st = new StringTokenizer(Cords);
			String x1 = st.nextToken();
			int x = Integer.parseInt(x1);
			String y1 = st.nextToken();
			int y = Integer.parseInt(y1);
			String z1 = st.nextToken();
			int z = Integer.parseInt(z1);
		
			//Common character information
			SystemMessage sm = new SystemMessage(614);
			sm.addString("Admin is teleporting you.");
			player.sendPacket(sm);
			
			TeleportToLocation tele = new TeleportToLocation(player, x,y,z);
			player.sendPacket(tele);
			player.setX(x);
			player.setY(y);
			player.setZ(z);

			//Admin information	
			SystemMessage smA = new SystemMessage(614);
			smA.addString("Character " + player.getName() + " teleported to "+ x + " " + y + " "+z);
			activeChar.sendPacket(smA);
		}
		
		showMainPage(client); //Back to start
	}	
	
	private void teleportToCharacter(ClientThread client)
	{
		L2PcInstance activeChar = client.getActiveChar();
		L2PcInstance player = L2World.getInstance().getPlayer(_characterToManipulate);
		
		if (player.getName().equals(activeChar.getName()))
		{	
			SystemMessage sm = new SystemMessage(614);
			sm.addString("You cannot self teleport.");
			activeChar.sendPacket(sm);
		}
		else
		{
			int x = player.getX();
			int y = player.getY();
			int z = player.getZ();
		
			TeleportToLocation tele = new TeleportToLocation(activeChar, x,y,z);
			activeChar.sendPacket(tele);
			activeChar.setX(x);
			activeChar.setY(y);
			activeChar.setZ(z);
		
			SystemMessage sm = new SystemMessage(614);
			sm.addString("You have teleported to character " + player.getName() + ".");
			activeChar.sendPacket(sm);
		}
		
		showMainPage(client); //Back to start
	}

	private void spawnMonster(ClientThread client, String monsterId)
	{	
		L2PcInstance activeChar = client.getActiveChar();
		
		int monsterTemplate = Integer.parseInt(monsterId);
		
		L2Npc template1 = NpcTable.getInstance().getTemplate(monsterTemplate);
		if (template1 == null)
		{
			return;
		}
		
		try
		{
			L2MonsterInstance mob = new L2MonsterInstance(template1);
			L2NpcData template2 = NpcDataTable.getInstance().getTemplate(monsterTemplate);
	
			L2Spawn spawn = new L2Spawn(template1, template2);
			spawn.setLocx(activeChar.getX());
			spawn.setLocy(activeChar.getY());
			spawn.setLocz(activeChar.getZ());
			spawn.setRandomx(0);
			spawn.setRandomy(0);
			spawn.setAmount(1);
			spawn.setHeading(activeChar.getHeading());
			spawn.setRespawnDelay(15);
			SpawnTable.getInstance().addNewSpawn(spawn);
			spawn.init();
		
			SystemMessage sm = new SystemMessage(614);
			sm.addString("You created a new spawn. " +mob.getName());
			activeChar.sendPacket(sm);
		}
		catch (Exception e)
		{
			SystemMessage sm = new SystemMessage(614);
			sm.addString("Problem while creating new spawn. ");
			activeChar.sendPacket(sm);
		}
		
		showHelpPage(client, "spawns.htm");
	}
	
	private void serverShutdown(ClientThread client, int seconds)
	{
		L2PcInstance activeChar = client.getActiveChar();
		L2PcInstance[] players = L2World.getInstance().getAllPlayers();
		
		if (seconds==0)
		{	
			NpcHtmlMessage adminReply = new NpcHtmlMessage(5);
		
			StringBuffer replyMSG = new StringBuffer("<html><title>Shutdown Server</title>");
			replyMSG.append("<body><br>");
		
			replyMSG.append("Enter in seconds the time till the server<p> shutdowns bellow:");
			replyMSG.append("<edit var=\"shutdown_time\" width=110>");
			replyMSG.append("<button value=\"Shutdown\" action=\"bypass -h admin_server_shutdown $shutdown_time\" width=80 height=15 back=\"sek.cbui94\" fore=\"sek.cbui92\">");			
			
			replyMSG.append("<br>");
			replyMSG.append("<center><button value=\"Back\" action=\"bypass -h admin_show\" width=40 height=15></center>");		
			replyMSG.append("</body></html>");
			
			adminReply.setHtml(replyMSG.toString());
			activeChar.sendPacket(adminReply);				
		}
		else
		{
			try 
			{
				for (int i=seconds; i==0; i--)
				{
					broadcastToAll("The server will shutdown soon.", activeChar.getName());
					
					if (seconds>0)
					{	
						int delay = 1000; //milliseconds
						
						Thread.sleep(1000);
		
				      	if (seconds==240)
						{
							broadcastToAll("The server will shutdown in 4 minutes", activeChar.getName());
						}
				      	else if (seconds==180)
						{
							broadcastToAll("The server will shutdown in 3 minutes", activeChar.getName());
						}					
						else if (seconds==120)
						{
							broadcastToAll("The server will shutdown in 2 minutes", activeChar.getName());
						}					
						else if (seconds==60)
						{
							broadcastToAll("The server will shutdown in 1 minutes", activeChar.getName());
						}
						else if (seconds==30)
						{
							broadcastToAll("The server will shutdown in 30 seconds", activeChar.getName());
						}
					}
					else
					{
						//TODO Shutdown Server
					}
				}
			} 
			catch (InterruptedException e)
			{
				// this will never happen
			}
		}
	}
		
	private void showHelpPage(ClientThread client, String filename)
	{
		File file = new File("data/html/admin/" + filename);
		FileInputStream fis = null;
		
		try
		{
			fis = new FileInputStream(file);
			byte[] raw = new byte[fis.available()];
			fis.read(raw);
			
			String content = new String(raw, "UTF-8");
			
			L2PcInstance activeChar = client.getActiveChar();			
			NpcHtmlMessage adminReply = new NpcHtmlMessage(5);

			adminReply.setHtml(content);
			activeChar.sendPacket(adminReply);
		}
		catch (Exception e)
		{
			// problem with adminhelp is ignored
		}		
		finally
		{
			try
			{
				fis.close();
			}
			catch (Exception e1)
			{
				// problems ignored
			}
		}
	}
	
	private void broadcastToAll(String message, String gmName)
	{
		L2PcInstance[] players = L2World.getInstance().getAllPlayers();
		CreatureSay cs = new CreatureSay(0, 9, "[GM]"+ gmName, message);
		
		for(int x=0;x<players.length;x++)
		{
			players[x].sendPacket(cs);
		}
	}
	
	public void showSkill(ClientThread client, String val)
	{
		L2PcInstance activeChar = client.getActiveChar();	
		
		int skillid = Integer.parseInt(val);
		L2Skill skill = SkillTable.getInstance().getInfo(skillid, 1);

		if (skill != null)
		{
			if (skill.getTargetType() == 0)
			{
				activeChar.setTarget(activeChar);
		
				MagicSkillUser msk = new MagicSkillUser(activeChar, skillid, 1, skill.getSkillTime() , skill.getReuseDelay());
				activeChar.sendPacket(msk);
				activeChar.broadcastPacket(msk);
		
				ActionFailed af = new ActionFailed();
				_log.fine("showing self skill, id: "+skill.getId()+" named: "+skill.getName());
			}
			else if (skill.getTargetType() == 1)
			{
				ActionFailed af = new ActionFailed();
				_log.fine("showing attack skill, id: "+skill.getId()+" named: "+skill.getName());				
			}
		}
		else
		{
		_log.fine("no such skill id: "+skillid);
		ActionFailed af = new ActionFailed();
		}
	}
}