/*
 * $Header: /var/lib/gforge/chroot/cvsroot/nextg/CVSROOT/L2_Gameserver/java/net/sf/l2j/gameserver/NpcTable.java,v 1.3 2004/08/05 00:53:40 nextg Exp $
 *
 * $Author: nextg $
 * $Date: 2004/08/05 00:53:40 $
 * $Revision: 1.3 $
 * $Log: NpcTable.java,v $
 * Revision 1.3  2004/08/05 00:53:40  nextg
 * RuSh
 *
 * Revision 1.5  2004/07/13 23:15:12  l2chef
 * empty blocks commented
 *
 * Revision 1.4  2004/07/04 11:08:08  l2chef
 * logging is used instead of system.out
 *
 * Revision 1.3  2004/06/27 20:50:02  l2chef
 * better error message when data file is missing. skipping of comment lines
 *
 * Revision 1.2  2004/06/27 08:51:43  jeichhorn
 * Added copyright notice
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.gameserver;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.LineNumberReader;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.logging.Logger;

import net.sf.l2j.gameserver.templates.L2Npc;

/**
 * This class ...
 * 
 * @version $Revision: 1.3 $ $Date: 2004/08/05 00:53:40 $
 */
public class NpcTable
{
	private static Logger _log = Logger.getLogger(NpcTable.class.getName());
	
	private static NpcTable _instance;
	
	private HashMap _npcs;
	
	public static NpcTable getInstance()
	{
		if (_instance == null)
		{
			_instance = new NpcTable();
		}
		return _instance;
	}
	
	private NpcTable()
	{
		_npcs = new HashMap();
		LineNumberReader lnr = null;
		try
		{
			File skillData = new File("data/npc.csv");
			lnr = new LineNumberReader(new BufferedReader( new FileReader(skillData)));
			
			String line = null;
			while ( (line=lnr.readLine()) != null)
			{
				if (line.trim().length() == 0 || line.startsWith("#"))
				{
					continue;
				}

				L2Npc npc = parseList(line);
				_npcs.put(new Integer(npc.getNpcId()), npc);
			}

			_log.config("created " + _npcs.size() + " NPC templates");
		}
		catch (FileNotFoundException e)
		{
			_log.warning("npc.csv is missing in data folder");
		}
		catch (Exception e)
		{
			_log.warning("error while creating npc table "+e);
		}
		finally
		{
			try
			{
				lnr.close();
			}
			catch (Exception e1)
			{
				// ignore problems
			}
		}
	}
	
	private L2Npc parseList(String line)
	{
		StringTokenizer st = new StringTokenizer(line,";");

		L2Npc npc = new L2Npc();
		int id = Integer.parseInt(st.nextToken());
		if (id > 1000000)
		{
			id -= 1000000;
		}
		npc.setNpcId(id);
		npc.setName( st.nextToken() );
		npc.setType( st.nextToken() );
		npc.setRadius( Double.parseDouble(st.nextToken()) );
		npc.setHeight( Double.parseDouble(st.nextToken()) );
		
		return npc;
	}

	/**
	 * @param template id
	 * @return
	 */
	public L2Npc getTemplate(int id)
	{
		return (L2Npc) _npcs.get(new Integer(id));
	}
}
