/*
 * $Header: /var/lib/gforge/chroot/cvsroot/nextg/CVSROOT/L2_Gameserver/java/net/sf/l2j/gameserver/clientpackets/RequestBuyItem.java,v 1.3 2004/08/05 00:53:40 nextg Exp $
 *
 * $Author: nextg $
 * $Date: 2004/08/05 00:53:40 $
 * $Revision: 1.3 $
 * $Log: RequestBuyItem.java,v $
 * Revision 1.3  2004/08/05 00:53:40  nextg
 * RuSh
 *
 * Revision 1.5  2004/07/11 22:43:09  l2chef
 * update inventory does not work right, so revert to old code
 *
 * Revision 1.4  2004/07/09 20:15:49  l2chef
 * less traffic by using update packet instead of full list. contributed by  whatev
 *
 * Revision 1.3  2004/07/04 11:12:33  l2chef
 * logging is used instead of system.out
 *
 * Revision 1.2  2004/06/27 08:51:42  jeichhorn
 * Added copyright notice
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.gameserver.clientpackets;

import java.io.IOException;
import java.util.logging.Logger;

import net.sf.l2j.gameserver.ClientThread;
import net.sf.l2j.gameserver.Connection;
import net.sf.l2j.gameserver.ItemTable;
import net.sf.l2j.gameserver.TradeController;
import net.sf.l2j.gameserver.model.L2BuyList;
import net.sf.l2j.gameserver.model.L2ItemInstance;
import net.sf.l2j.gameserver.model.L2PcInstance;
import net.sf.l2j.gameserver.serverpackets.ItemList;
import net.sf.l2j.gameserver.serverpackets.SystemMessage;

/**
 * This class ...
 * 
 * @version $Revision: 1.3 $ $Date: 2004/08/05 00:53:40 $
 */
public class RequestBuyItem extends ClientBasePacket
{
	private static final String _C__1F_REQUESTBUYITEM = "[C] 1F RequestBuyItem";
	private static Logger _log = Logger.getLogger(RequestBuyItem.class.getName());

	/**
	 * packet type id 0x1f
	 * 
	 * sample
	 * 
	 * 1f
	 * 44 22 02 01		// list id
	 * 02 00 00 00		// items to buy
	 * 
	 * 27 07 00 00		// item id
	 * 06 00 00 00		// count
	 * 
	 * 83 06 00 00
	 * 01 00 00 00
	 * 
	 * format:		cdd (dd) 
	 * @param decrypt
	 */
	public RequestBuyItem(byte[] decrypt, ClientThread client) throws IOException
	{
		super(decrypt);
		int listId = readD();
		int count = readD();
		L2ItemInstance[] items = new L2ItemInstance[count];
		for (int i=0;i<count;i++)
		{
			int itemId = readD();
			int cnt = readD();
			L2ItemInstance inst = ItemTable.getInstance().createItem(itemId);
			inst.setCount(cnt);
			items[i] = inst;
		}
		


		//TODO check if valid buylist, stackable items ?
		L2PcInstance activeChar = client.getActiveChar();
		Connection con = client.getConnection();
		
		int neededMoney = 0;
		int currentMoney = activeChar.getAdena(); 
		L2BuyList list = TradeController.getInstance().getBuyList(listId);

		for (int i = 0; i < items.length; i++)
		{
			int count2 = items[i].getCount();
			int id = items[i].getItemId();
			int price = list.getPriceForItemId(id);
			if (price == -1)
			{
				_log.warning("ERROR, no price found .. wrong buylist ??");

			}
			neededMoney += count2 * price;
		}

		if (neededMoney > currentMoney)
		{
			SystemMessage sm = new SystemMessage(279);
			con.sendPacket(sm);
			return;
		}

		activeChar.reduceAdena(neededMoney);
		for (int i = 0; i < items.length; i++)
		{
			activeChar.getInventory().addItem(items[i]);
		}

		//TODO only send Inventory update instead of full list !
		ItemList il = new ItemList(activeChar, false);
		con.sendPacket(il);
	}

	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.clientpackets.ClientBasePacket#getType()
	 */
	public String getType()
	{
		return _C__1F_REQUESTBUYITEM;
	}
}
