/*
 * $Header: /var/lib/gforge/chroot/cvsroot/nextg/CVSROOT/L2_Gameserver/java/net/sf/l2j/gameserver/clientpackets/RequestMagicSkillUse.java,v 1.3 2004/08/05 00:53:40 nextg Exp $
 *
 * $Author: nextg $
 * $Date: 2004/08/05 00:53:40 $
 * $Revision: 1.3 $
 * $Log: RequestMagicSkillUse.java,v $
 * Revision 1.3  2004/08/05 00:53:40  nextg
 * RuSh
 *
 * Revision 1.4  2004/07/13 23:02:11  l2chef
 * empty blocks commented
 *
 * Revision 1.3  2004/07/04 11:12:33  l2chef
 * logging is used instead of system.out
 *
 * Revision 1.2  2004/06/27 08:51:42  jeichhorn
 * Added copyright notice
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.gameserver.clientpackets;

import java.io.IOException;
import java.util.logging.Logger;

import net.sf.l2j.gameserver.ClientThread;
import net.sf.l2j.gameserver.Connection;
import net.sf.l2j.gameserver.SkillTable;
import net.sf.l2j.gameserver.model.L2Character;
import net.sf.l2j.gameserver.model.L2Skill;
import net.sf.l2j.gameserver.serverpackets.ActionFailed;
import net.sf.l2j.gameserver.serverpackets.MagicEffectIcons;
import net.sf.l2j.gameserver.serverpackets.MagicSkillLaunched;
import net.sf.l2j.gameserver.serverpackets.MagicSkillUser;
import net.sf.l2j.gameserver.serverpackets.MoveToPawn;
import net.sf.l2j.gameserver.serverpackets.SetupGauge;
import net.sf.l2j.gameserver.serverpackets.StatusUpdate;
import net.sf.l2j.gameserver.serverpackets.SystemMessage;

/**
 * This class ...
 * 
 * @version $Revision: 1.3 $ $Date: 2004/08/05 00:53:40 $
 */
public class RequestMagicSkillUse extends ClientBasePacket
{
	private static final String _C__2F_REQUESTMAGICSKILLUSE = "[C] 2F RequestMagicSkillUse";
	private static Logger _log = Logger.getLogger(RequestMagicSkillUse.class.getName());

	/**
	 * packet type id 0x2f
	 * format:		cddc
	 * @param rawPacket
	 */
	public RequestMagicSkillUse(byte[] rawPacket, ClientThread client) throws IOException
	{
		super(rawPacket);
		int magicId  = readD();
		int data2  = readD();
		int data3 =  readC();
		
		_log.fine("SkillId:"+magicId+" SkillLevel:"+data2+" data3:"+data3);
		
		L2Character activeChar = client.getActiveChar();
		Connection con = client.getConnection();
		
		

		int level = 1;
		L2Skill skill = SkillTable.getInstance().getInfo(magicId, level);
		if (skill.isPassive())
		{
			// just ignore the passive skill request. why does the client send it anyway ??
			return;
		}
		
		if (skill.getMpConsume() > activeChar.getCurrentMp())
		{
			SystemMessage sm = new SystemMessage(SystemMessage.NOT_ENOUGH_MP);
			con.sendPacket(sm);
			return;
		}
			

		if (skill.getCastRange() > 1 && activeChar.getTarget() != null)
		{
			MoveToPawn move = new MoveToPawn(activeChar,  activeChar.getTarget(), skill.getCastRange());
			con.sendPacket(move);
		}
		
		ActionFailed af = new ActionFailed();
		con.sendPacket(af);

		
		MagicSkillUser msk = new MagicSkillUser(activeChar, magicId, level, skill.getSkillTime(), skill.getReuseDelay());
		con.sendPacket(msk);

		SetupGauge sg = new SetupGauge(0, skill.getSkillTime());
		con.sendPacket(sg);

		SystemMessage sm = new SystemMessage(0x2e);
		sm.addSkillName(magicId);
		con.sendPacket(sm);


		if (skill.getSkillTime() > 200)
		{
			try
			{
				Thread.sleep(skill.getSkillTime()-200);
			}
			catch (InterruptedException e)
			{
				// ignore interruptions
			}

			// this must be send before the skill time ends.
			// most spells default to "failed" if this is not received in time
			MagicSkillLaunched msl = new MagicSkillLaunched(activeChar, magicId, level);
			con.sendPacket(msl);
		}

		
		StatusUpdate su = new StatusUpdate(activeChar.getObjectId());
		activeChar.reduceCurrentMp(skill.getMpConsume());
		su.addAttribute(StatusUpdate.CUR_MP, (int) activeChar.getCurrentMp());
		con.sendPacket(su);
		

//		L2NpcInstance target = (L2NpcInstance) L2World.getInstance().findObject(activeChar.getTargetId());
//		TargetSelected ts = new TargetSelected(target.getObjectId(), activeChar.getObjectId(), activeChar.getX(), activeChar.getY(), activeChar.getZ());
//		con.sendPacket(ts);
		
//		ChangeMoveType cmt = new ChangeMoveType(target,ChangeMoveType.RUN);
//		con.sendPacket(cmt);
		
//		target.moveTo(activeChar.getX(), activeChar.getY(), activeChar.getZ(), 0);
		
		if (skill.getOperateType() == L2Skill.OP_DURATION)
		{	
			MagicEffectIcons mei = new MagicEffectIcons();
			mei.addEffect(magicId, level, skill.getBuffDuration());
			con.sendPacket(mei);
		}

		sm = new SystemMessage(0x6e);
		sm.addSkillName(magicId);
		con.sendPacket(sm);		
	}

	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.clientpackets.ClientBasePacket#getType()
	 */
	public String getType()
	{
		return _C__2F_REQUESTMAGICSKILLUSE;
	}
}
