package net.sf.l2j.gameserver.model;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Logger;

import net.sf.l2j.gameserver.ItemTable;
import net.sf.l2j.gameserver.serverpackets.DropItem;
import net.sf.l2j.gameserver.templates.L2Npc;
import net.sf.l2j.gameserver.templates.L2Weapon;
import net.sf.l2j.gameserver.templates.L2Item;

/**
 * This class ...
 * 
 * @version $Revision: 1.3 $ $Date: 2004/08/05 00:53:40 $
 */
public class L2Attackable extends L2NpcInstance
{
	private static Logger _log = Logger.getLogger(L2Attackable.class.getName());
	
	class AITask extends TimerTask
	{
		L2Character _instance;
		
		public AITask(L2Character instance)
		{
			_instance = instance;
		}
		
		public void run()
		{
			try
			{
				int x1 = getX() + getRnd().nextInt(500) -250;
				int y1 = getY() + getRnd().nextInt(500) -250;
							
				moveTo(x1, y1, getZ(), 0);
			}
			catch (Exception e)
			{
				_log.warning(getObjectId()+": Problem occured in AiTask:"+e);
			}
		}
	}

	class AIAttackeTask extends TimerTask
	{
		L2Character _instance;
		
		public AIAttackeTask(L2Character instance)
		{
			_instance = instance;
		}
		
		public void run()
		{
			try
			{
				if (!isInCombat())
				{
					_log.finer(getObjectId()+": monster knows "+getKnownPlayers().size() + " players");
					
					Set knownPlayers = getKnownPlayers();
					for (Iterator iter = knownPlayers.iterator(); iter.hasNext();)
					{
						L2PcInstance player = (L2PcInstance) iter.next();
						
						// dont do further checking if the player is dead or Z coordinate is very different
						// TODO find a good value for the max allowed Z distance
						if (getCondition2(player)) 
						{
						
							double distance = getDistance(player.getX(), player.getY());

							if((distance <= (getCollisionRadius()+200)))
							{
								_log.fine(getObjectId()+": Player "+player.getObjectId()+" in aggro range. Attacking!");
								
								if(_currentAiTask != null)
								{	
									_currentAiTask.cancel();
								}
								
								setTarget(player);
								startAttack(player);
							}
						}
					}				  					
				}
				else
				{
					if(_currentAIAttackeTask != null)
					{
						_currentAIAttackeTask.cancel();
					}
				}
			}
			catch (Exception e)
			{
				_log.warning(getObjectId()+": Problem occured in AiAttackTask:"+e);
			}
		}
	}

	private int _moveRadius;
	private boolean _active;
	private AITask _currentAiTask;
	private AIAttackeTask _currentAIAttackeTask;
	
	private static Timer _aiTimer = new Timer(true);
	private static Timer _attackTimer = new Timer(true);
	
	private HashMap _aggroList = new HashMap();
	private L2Weapon _dummyWeapon; 
	
	/**
	 * @param template
	 */
	public L2Attackable(L2Npc template)
	{
		super(template);
	}
		
	public boolean getCondition2(L2PcInstance player)
	{
		return false;
	}
	
	/**
	 * event that is called when the destination coordinates are reached
	 */
	public void onTargetReached()
	{	
		super.onTargetReached();

		switch (getCurrentState())
		{
			case STATE_PICKUP_ITEM:
//					doPickupItem();
				break;
			case STATE_ATTACKING:
				startCombat();
				break;
			case STATE_RANDOM_WALK:
				randomWalk();
				break;
			default:
				break;
		}
	}
	

	private void randomWalk()
	{
		if (_active)
		{
			_log.fine(getObjectId()+": target reached ! new target calculated.");
			int wait = (10 + getRnd().nextInt(120))*1000;
			
			_currentAiTask = new AITask(this);
			_aiTimer.schedule(_currentAiTask, wait);
		}
		else
		{
			_log.fine(getObjectId()+": target reached ! noone around .. cancel movement.");
		}
	}

	protected void startRandomWalking()
	{
		_currentAiTask = new AITask(this);
		
		int wait = (10 + getRnd().nextInt(120))*1000;			
		_aiTimer.schedule(_currentAiTask, wait);
		setCurrentState(STATE_RANDOM_WALK);
	}
	
	protected void startTargetScan()
	{
		_currentAIAttackeTask = new AIAttackeTask(this);
		_attackTimer.scheduleAtFixedRate(_currentAIAttackeTask,100,50);
	}

	public void startAttack(L2Character target)
	{
		// one target is enough
		stopTargetScan();
		super.startAttack(target);
	}
	
	
	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Object#setX(int)
	 */
	public void setX(int x)
	{
		super.setX(x);
	}
	
	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Object#setY(int)
	 */
	public void setY(int y)
	{
		super.setY(y);
	}
	
	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Object#setZ(int)
	 */
	public void setZ(int z)
	{
		super.setZ(z);
	}
		
	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Character#removeKnownObject(net.sf.l2j.gameserver.model.L2Object)
	 */
	public void removeKnownObject(L2Object object)
	{
		super.removeKnownObject(object);

		// this monster forgets any agro when hostile leaves the area
		if (object instanceof L2Character)
		{
			_aggroList.remove((L2Character) object);
		}
	}

	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Character#reduceCurrentHp(int)
	 */
	public void reduceCurrentHp(int damage, L2Character attacker)
	{
		super.reduceCurrentHp(damage, attacker);
		// this is usually only called in combat

		calculateAggro(damage, attacker);
		
		if (!isDead() && attacker != null)
		{
			if (!isInCombat())
			{
				stopRandomWalking();
				
				startAttack(attacker);
			}
			else
			{
				_log.fine("already attacking");
			}
		}
		else if (isDead())
		{
		    stopRandomWalking();
		    stopTargetScan();

		    calculateRewards(attacker);
	    }
	}
	

	/**
	 * @param attacker
	 */
	private void calculateRewards(L2Character lastAttacker)
	{
		Iterator it = _aggroList.keySet().iterator();
		int numberOfAttackers = _aggroList.size();
		
		while (it.hasNext())
		{
		    Object att = it.next();
		    if (att instanceof net.sf.l2j.gameserver.model.L2PcInstance)
		    {
		    	L2PcInstance attacker = (L2PcInstance) att;
		    	Integer value = (Integer) _aggroList.get(attacker);
		        
		        int npcID = getNpcTemplate().getNpcId();
		        int npcLvl = getLevel();
		        
		        _log.fine("Player "+lastAttacker.getName()+" Killed npc id: "+npcID+" npc level: "+npcLvl);
		        if (npcID > 0 )
		        {
		        	int newXp = 0;
		        	int newSp = 0;
		        	
		        	if (attacker.getLevel() <= npcLvl)
		        	{
		        		newXp = getExpReward()/numberOfAttackers;
		        		newSp = getSpReward()/numberOfAttackers;
		        	}
		        	else if (attacker.getLevel() > npcLvl)
		        	{
		        		int diff = attacker.getLevel()- getLevel();
		        		newXp = getExpReward() - (9 * diff) * getExpReward() / 100;
		        		newSp = getSpReward() - (9 * diff) * getSpReward() / 100;
		        	
		        		if (newXp <= 0)
		        		{
		        			newXp = 0;
		        			newSp = 0;
		        		}
		        		else if (newSp <= 0 && newXp > 0)
		        		{
		        			newSp = 0;
		        		}
			        }
		        	
        			attacker.addExpAndSp(newXp, newSp);
		        }
		    }
		}
		
		//FIXME add a loottable to the monsters and find out if something droped
		calculateAdenaDrop(lastAttacker);
		calculateItemDrop(lastAttacker);
	}

	/**
	 * @param damage
	 * @param attacker
	 */
	private void calculateAggro(int damage, L2Character attacker)
	{
		int newAggro = damage;
		Integer aggroValue = (Integer) _aggroList.get(attacker);
		if (aggroValue != null)
		{
			// add the old value
			newAggro += aggroValue.intValue();
		}
		
		_aggroList.put(attacker, new Integer(newAggro));
		
		if (_aggroList.size() == 1)
		{
			setTarget(attacker);
		}
		else
		{
			// find highest aggrovalue to decide target should be here
			// now we just use the last attacker is the new target
			setTarget(attacker);
		}
	}

	/**
	 * @param attacker
	 */
	private void calculateAdenaDrop(L2Character attacker)
	{
		int money = getRnd().nextInt(100);
		_log.fine("Drop adena (if lower then 30 yes): "+money);
		if (money <30)
		{
		    int npcID = getNpcTemplate().getNpcId();
		    if (attacker.getLevel() > getLevel()+9)
		    {
		    	_log.fine("Dropping no money, player too high lvl");	
		    }
		    else
		    {
				// this is here just to make the monster drop something for now
				L2ItemInstance drop = ItemTable.getInstance().createItem(57);
				drop.setCount(money + getLevel() * 5);
				_log.fine("Dropped: "+(money + getLevel()* 5)+" Adena");
				// position where mob was last
				drop.setX(getX());
				drop.setY(getY());
				drop.setZ(getZ());
		    
				drop.setOnTheGround(true);
				DropItem di = new DropItem(drop, getObjectId());
	   			// also notify all other players about the drop
	   			L2Character[] players = broadcastPacket(di);
	   			for (int i = 0; i < players.length; i++)
	   			{
	   				((L2PcInstance)players[i]).addKnownObjectWithoutCreate(drop);
	   			}
				
				L2World.getInstance().addVisibleObject(drop);
		    }
		}
	}

	/**
	 * @param attacker
	 */
	private void calculateItemDrop(L2Character attacker)
	{
		int dropitem = getRnd().nextInt(4230);
		int chanceitemdrop = getRnd().nextInt(1000);
		if (chanceitemdrop <50 & dropitem >0)
		{
			// this is here just to make the monster drop something for now
			L2PcInstance l2pc = (L2PcInstance) attacker;
		    int npcID = getNpcTemplate().getNpcId();
			
			L2Item thisitem = ItemTable.getInstance().getTemplate(dropitem);
		    if (attacker.getLevel() > getLevel()+9)
		    {
		    	_log.fine("Dropping no item, player too high lvl");	
		    }
		    else
		    {
		    	if (thisitem != null)
		    	{
		    		_log.fine("Item id to drop: "+dropitem);
		    		L2ItemInstance dropit = ItemTable.getInstance().createItem(dropitem);
		    		// position where mob was last
		    		dropit.setX(getX());
		    		dropit.setY(getY());
		    		dropit.setZ(getZ());
		    		
		    		// otherwise you cannot pick it up
		    		dropit.setOnTheGround(true);
		   			DropItem dis = new DropItem(dropit,getObjectId());
		   			
		   			// also notify all other players about the drop
		   			L2Character[] players = broadcastPacket(dis);
		   			for (int i = 0; i < players.length; i++)
		   			{
		   				((L2PcInstance)players[i]).addKnownObjectWithoutCreate(dropit);
		   			}
		   			
					L2World.getInstance().addVisibleObject(dropit);
		   			
		    	}
			}
		}
	}

	/**
	 * 
	 */
	protected void stopRandomWalking()
	{
		// dont walk around any more
		if (_currentAiTask != null)
		{
			_currentAiTask.cancel();
			_currentAiTask = null;
		}
	}

	protected boolean isTargetScanActive()
	{
		return (_currentAIAttackeTask != null);		
	}
	
	protected void stopTargetScan()
	{
		if (_currentAIAttackeTask != null)
		{
			_currentAIAttackeTask.cancel();
			_currentAIAttackeTask = null;
		}
	}
	
	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Character#setCurrentHp(double)
	 */
	public void setCurrentHp(double currentHp)
	{
		super.setCurrentHp(currentHp);
	}
	
	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Character#getActiveWeapon()
	 */
	public L2Weapon getActiveWeapon()
	{
		return _dummyWeapon;
	}

	/**
	 * this method has to be called AFTER setLevel() was executed
	 */
	public void setPhysicalAttack(int physicalAttack)
	{
		super.setPhysicalAttack(physicalAttack);

		// create a dummy weapon 
		_dummyWeapon = new L2Weapon();
		_dummyWeapon.setPDamage(physicalAttack);
		int randDmg = getLevel();
		_dummyWeapon.setRandomDamage(randDmg);
	}
	
	//some exports for guards
	public boolean noTarget()
	{
		return _aggroList.isEmpty();
	}

	public boolean containsTarget(L2Character player)
	{
		return _aggroList.containsKey(player);
	}

	public void clearAggroList()
	{
		_aggroList.clear();
	}
	
	public boolean isActive()
	{
		return _active;
	}
	
	public void setActive(boolean b)
	{
		_active = b;
	}
	
	public void setMoveRadius(int i)
	{
		_moveRadius = i;
	}
	
}