package net.sf.l2j.gameserver.model;

import java.util.logging.Logger;

import net.sf.l2j.gameserver.serverpackets.MyTargetSelected;
import net.sf.l2j.gameserver.serverpackets.SetToLocation;
import net.sf.l2j.gameserver.templates.L2Npc;

/**
 * This class ...
 *  
 */
public class L2GuardInstance extends L2Attackable
{

	private int _homeX;
	private int _homeY;
	private int _homeZ;
	private boolean _hasHome;

	private static Logger _log = Logger.getLogger(L2GuardInstance.class.getName());

	public L2GuardInstance(L2Npc template)
	{
		super(template);
		setCurrentState(STATE_IDLE);
	}

	/*
	 * ******************************************** PK stuff
	 * ********************************************
	 */

	
	public void addKnownObject(L2Object object)
	{
		if (!_hasHome) getHomeLocation();
		super.addKnownObject(object);
		if (object instanceof L2PcInstance) 
		{
			L2PcInstance player = (L2PcInstance) object;
			if ( (player.getKarma() > 0) && !isTargetScanActive())
			{
				_log.fine(getObjectId()+": PK "+player.getObjectId()+" entered scan range");
				startTargetScan();
			}
		}
	}		


	public void removeKnownObject(L2Object object)
	{
		super.removeKnownObject(object);
		
		if (getKnownPlayers().isEmpty())
		{
			setActive(false);
			clearAggroList();
			removeAllKnownObjects();
			stopTargetScan();
		}
		
		if (noTarget()) returnHome();
	}

	private void getHomeLocation()
	{
		_hasHome = true;
		_homeX = getX();
		_homeY = getY();
		_homeZ = getZ();

		_log.fine(getObjectId()+": Home location set to X:" + _homeX + " Y:" + _homeY + " Z:"
				+ _homeZ);
	}

	private void returnHome()
	{
		_log.fine(getObjectId()+": moving home");
		clearAggroList();
		moveTo(_homeX, _homeY, _homeZ, 0);
	}
	
	public boolean getCondition2(L2PcInstance player)
	{
		return (!player.isDead() && (player.getKarma() >0) && (Math.abs(getZ()-player.getZ()) <= 100 ));
	}


	/*
	 * ******************************************** Chat & Quest stuff
	 * ********************************************
	 */

	
	public String getHtmlPath(int npcId, int val)
	{
		String pom = "";
		if (val == 0)
		{
			pom = "" + npcId;
		} 
		else 
		{
			pom = npcId + "-" + val;
		}
		return "data/html/guard/" + pom + ".htm";
	}

	public void onAction(L2PcInstance player)
	{
		if (getObjectId() != player.getTargetId())
		{
			player.setCurrentState(L2Character.STATE_IDLE);
			_log.fine(player.getObjectId()+": Targetted guard "+getObjectId());
			player.setTarget(this);
			MyTargetSelected my = new MyTargetSelected(getObjectId(), 0);
			player.sendPacket(my);

			// correct location
			player.sendPacket(new SetToLocation(this));
		}
		else
		{
			if (containsTarget((L2Character) player))
			{
				_log.fine(player.getObjectId()+": Attacked guard "+getObjectId());
				player.startAttack(this);
			} else
			{
				showChatWindow(player, 0);
			}
		}
	}
}