/*
 * $Header: /var/lib/gforge/chroot/cvsroot/nextg/CVSROOT/L2_Gameserver/java/net/sf/l2j/gameserver/model/L2NpcInstance.java,v 1.3 2004/08/05 00:53:40 nextg Exp $
 *
 * $Author: nextg $
 * $Date: 2004/08/05 00:53:40 $
 * $Revision: 1.3 $
 * $Log: L2NpcInstance.java,v $
 * Revision 1.3  2004/08/05 00:53:40  nextg
 * RuSh
 *
 * Revision 1.14  2004/07/23 02:23:21  l2chef
 * npc chat dialogs unified (NuocNam)
 *
 * Revision 1.13  2004/07/23 01:47:51  l2chef
 * spawn handler is notified when npc dies
 *
 * Revision 1.12  2004/07/19 02:06:45  l2chef
 * decay timer got moved here
 *
 * Revision 1.11  2004/07/18 17:56:41  l2chef
 * npcs change to running if they attack
 *
 * Revision 1.10  2004/07/18 17:38:38  l2chef
 * aggressive monsters scan for targets in range (FTPW)
 *
 * Revision 1.9  2004/07/17 23:17:41  l2chef
 * attackrange and forced attack added
 *
 * Revision 1.8  2004/07/11 22:12:50  l2chef
 * changed comment
 *
 * Revision 1.7  2004/07/11 11:48:20  l2chef
 * exp and sp values for NPCs added
 *
 * Revision 1.6  2004/07/08 22:22:57  l2chef
 * common stuff moved to L2NpcInstance
 *
 * Revision 1.5  2004/07/07 23:41:04  l2chef
 * new merchant conversation files (done by NightMarez)
 * some design changes
 *
 * Revision 1.4  2004/07/04 19:05:25  l2chef
 * show default response message when talking to npc
 *
 * Revision 1.3  2004/07/04 11:13:27  l2chef
 * logging is used instead of system.out
 *
 * Revision 1.2  2004/06/27 08:51:42  jeichhorn
 * Added copyright notice
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.gameserver.model;

import java.io.File;
import java.io.FileInputStream;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Logger;

import net.sf.l2j.gameserver.serverpackets.ActionFailed;
import net.sf.l2j.gameserver.serverpackets.ChangeMoveType;
import net.sf.l2j.gameserver.serverpackets.MyTargetSelected;
import net.sf.l2j.gameserver.serverpackets.NpcHtmlMessage;
import net.sf.l2j.gameserver.serverpackets.SetToLocation;
import net.sf.l2j.gameserver.serverpackets.StatusUpdate;
import net.sf.l2j.gameserver.templates.*;

/**
 * This class represents a Non-Player-Character in the world. it can be 
 * a monster or a friendly character.
 * it also uses a template to fetch some static values.
 * the templates are hardcoded in the client, so we can rely on them.
 * 
 * @version $Revision: 1.3 $ $Date: 2004/08/05 00:53:40 $
 */
public class L2NpcInstance extends L2Character
{
	private static Logger _log = Logger.getLogger(L2NpcInstance.class.getName());
	private L2Npc _npcTemplate;
	private boolean _attackable;
	private int _rightHandItem;
	private int _leftHandItem;
	
	private int _expReward;
	private int _spReward;
	private int _attackRange;
	
	private boolean _aggressive;
	private DecayTask _decayTask;
	private static Timer _decayTimer = new Timer(true);
	private L2Spawn _spawn;


	
	class DecayTask extends TimerTask
	{
		L2Character _instance;
		
		public DecayTask(L2Character instance)
		{
			_instance = instance;
		}
		
		public void run()
		{
			_instance.onDecay();
		}
	}
	
	public L2NpcInstance(L2Npc template)
	{
		_npcTemplate = template;
		setCollisionHeight(template.getHeight());
		setCollisionRadius(template.getRadius());
	}
	
	public boolean isAggressive()
	{
		return _aggressive;
	}
	
	public void startAttack(L2Character target)
	{
		if (!isRunning())
		{
			setRunning(true);
			ChangeMoveType move = new ChangeMoveType(this, ChangeMoveType.RUN);
			broadcastPacket(move);
		}
		
		super.startAttack(target);
	}
	
	public void setAggressive(boolean aggressive)
	{
		_aggressive = aggressive;
	}

	/**
	 * @return
	 */
	public L2Npc getNpcTemplate()
	{
		return _npcTemplate;
	}
	
	

	/**
	 * @return
	 */
	public boolean isAttackable()
	{
		return _attackable;
	}

	/**
	 * @param b
	 */
	public void setAttackable(boolean b)
	{
		_attackable = b;
	}

	/**
	 * @return
	 */
	public int getLeftHandItem()
	{
		return _leftHandItem;
	}

	/**
	 * @return
	 */
	public int getRightHandItem()
	{
		return _rightHandItem;
	}

	/**
	 * @param i
	 */
	public void setLeftHandItem(int i)
	{
		_leftHandItem = i;
	}

	/**
	 * @param i
	 */
	public void setRightHandItem(int i)
	{
		_rightHandItem = i;
	}
	
	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Object#onAction(net.sf.l2j.gameserver.model.L2PcInstance)
	 */
	public void onAction(L2PcInstance player)
	{
		if (getObjectId() != player.getTargetId())
		{
			player.setCurrentState(L2Character.STATE_IDLE);
			_log.fine("new target selected:"+getObjectId());
			player.setTarget(this);
			MyTargetSelected my = new MyTargetSelected(getObjectId(), player.getLevel() - getLevel());
			player.sendPacket(my);
			
			if (isAttackable())
			{	
				StatusUpdate su = new StatusUpdate(getObjectId());
				su.addAttribute(StatusUpdate.CUR_HP, (int)getCurrentHp() );
				su.addAttribute(StatusUpdate.MAX_HP, getMaxHp() );
				player.sendPacket(su);
			}
			
			// correct location
			player.sendPacket(new SetToLocation(this));
		}
		else
		{
			if (isAttackable())
			{
				player.startAttack(this);
			}
			else
			{
				showChatWindow(player, 0);
				player.sendPacket(new ActionFailed());
			}
		}
	}
	
	public void onActionShift(L2PcInstance player)
	{
		NpcHtmlMessage html = new NpcHtmlMessage(1);
		StringBuffer html1 = new StringBuffer("<html><title>Hello</title><body>Current target:<br>oid:");
		html1.append(""+getObjectId());
		html1.append("<br>template id:");
		html1.append(""+getNpcTemplate().getNpcId());
		
		html1.append("  hp:");
		html1.append(""+getCurrentHp());
		html1.append("  mp:");
		html1.append(""+getCurrentMp());
		html1.append("<br><font color=\"LEVEL\">level:</font>");
		html1.append(""+getLevel());
		html1.append("<br>impl:");
		html1.append(""+getClass().getName());
		html1.append("<br><a action=\"link dual_sword001.htm\">link</a>");
		html1.append("<br><a action=\"bypass -h talk_select\">talk select</a>");
		html1.append("<br><a action=\"bypass -h quest_accept?quest_id=205\">quest accept</a>");
		html1.append("<br><a action=\"bypass -h learn_skill\">learn skill</a>");
		html1.append("<br><a action=\"bypass learn_skill2\">learn skill2</a>");
		html1.append("</body></html>");
		
		html.setHtml(html1.toString());
		player.sendPacket(html);
		
		player.sendPacket(new ActionFailed());
	}

	/**
	 * this gets called by the RequestBypassToServer packet handler
	 * override it in subclasses to use a different behavior
	 * @param command
	 */
	public void onBypassFeedback(L2PcInstance player, String command)
	{
		if (command.startsWith("Quest"))
		{
			int val = Integer.parseInt(command.substring(6));
			showQuestWindow(player, val);
		}
		else if (command.startsWith("Chat"))
		{
			int val = Integer.parseInt(command.substring(5));
			showChatWindow(player, val);
		}
	}
	
	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Character#getActiveWeapon()
	 */
	public L2Weapon getActiveWeapon()
	{
		// regular NPCs dont have weapons, so they cant fight
		return null;
	}

	/**
	 * @param player
	 * @param content
	 */
	public void insertObjectIdAndShowChatWindow(L2PcInstance player, String content)
	{
		
		content = content.replaceAll("%objectId%", String.valueOf(getObjectId()));
		NpcHtmlMessage npcReply = new NpcHtmlMessage(5);
		
		npcReply.setHtml(content);
		player.sendPacket(npcReply);
	}

	protected void showQuestWindow(L2PcInstance player, int val)
	{
		_log.fine("Showing quest window");
		
		NpcHtmlMessage html = new NpcHtmlMessage(1);
		html.setHtml("<html><head><body>There is no quests here yet.</body></html>");
		player.sendPacket(html);
		
		player.sendPacket( new ActionFailed() );
	}

	
	public String getHtmlPath(int npcId, int val)
	{
		String pom = "";
		if (val == 0)
		{
			pom = "" + npcId;
		} 
		else 
		{
			pom = npcId + "-" + val;
		}
		
		String temp = "data/html/default/" + pom + ".htm"; 
		File mainText = new File(temp);
		if (mainText.exists())
		{
			return temp;
		}
		else
		{
			return "data/html/npcdefault.htm";
		}
	}

	public void showChatWindow(L2PcInstance player, int val)
	{
		int npcId = getNpcTemplate().getNpcId();
		
		String filename = (getHtmlPath(npcId, val));
		
		File file = new File(filename);
		FileInputStream fis = null;		
		try
		{
			fis = new FileInputStream(file);
			byte[] raw = new byte[fis.available()];
			fis.read(raw);
			
			String content = new String(raw, "UTF-8");
			
			insertObjectIdAndShowChatWindow(player, content);
			
		}
		catch (Exception e)
		{
			_log.warning("problem with npc text "+e);
		}
		finally
		{
			try
			{
				fis.close();
			}
			catch (Exception e1)
			{
				// ignore problem
			}
		}
		
		player.sendPacket( new ActionFailed() );
	}

	/**
	 * @param exp
	 */
	public void setExpReward(int exp)
	{
		_expReward = exp;
	}

	/**
	 * @param sp
	 */
	public void setSpReward(int sp)
	{
		_spReward = sp;
	}
	
	public int getExpReward()
	{
		return _expReward;
	}
	
	public int getSpReward()
	{
		return _spReward;
	}

	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Character#getAtkRange()
	 */
	public int getAttackRange()
	{
		return _attackRange; 
	}
	
	public void setAttackRange(int range)
	{
		_attackRange = range;
	}
	
	public void reduceCurrentHp(int i, L2Character attacker)
	{
		super.reduceCurrentHp(i, attacker);

		if (isDead())
		{
			// in 7 seconds this corpse will disappear
			_decayTask = new DecayTask(this);
			_decayTimer.schedule(_decayTask, 7000);
			// tell my creator that I'm gone
			_spawn.decreaseCount(_npcTemplate.getNpcId());
		}
	}

	/**
	 * @param spawn
	 */
	public void setSpawn(L2Spawn spawn)
	{
		_spawn = spawn;
	}
}
