/*
 * $Header: /var/lib/gforge/chroot/cvsroot/nextg/CVSROOT/L2_Gameserver/java/net/sf/l2j/gameserver/model/L2PcInstance.java,v 1.3 2004/08/05 00:53:40 nextg Exp $
 *
 * $Author: nextg $
 * $Date: 2004/08/05 00:53:40 $
 * $Revision: 1.3 $
 * $Log: L2PcInstance.java,v $
 * Revision 1.3  2004/08/05 00:53:40  nextg
 * RuSh
 *
 * Revision 1.21  2004/08/04 21:15:19  l2chef
 * new flag for items added to prevent duping
 *
 * Revision 1.20  2004/08/02 00:08:47  l2chef
 * better solution to drop item problem
 *
 * Revision 1.19  2004/07/30 00:16:34  l2chef
 * fix for dead respawn problem
 *
 * Revision 1.18  2004/07/29 20:09:20  l2chef
 * Pdef/Matk/Mdef updates added (Nightmarez/Nuocnam)
 *
 * Revision 1.17  2004/07/28 23:58:59  l2chef
 * patk updated and free skills are rewarded automatically (Nightmarez)
 *
 * Revision 1.16  2004/07/25 22:58:24  l2chef
 * pet system started (whatev)
 *
 * Revision 1.15  2004/07/25 00:38:09  l2chef
 * moving objects appear now as moving when they get into viewrange (NuocNam)
 *
 * Revision 1.14  2004/07/23 01:47:21  l2chef
 * all object spawn and delete is now handeld here
 *
 * Revision 1.13  2004/07/19 02:08:08  l2chef
 * postionListeners added
 * party code added
 *
 * Revision 1.12  2004/07/18 17:39:20  l2chef
 * action failed removed for case of successful combat start
 *
 * Revision 1.11  2004/07/17 23:14:52  l2chef
 * damage is now reduced before awarding exp
 * additional check if attack target is valid and range is ok
 *
 * Revision 1.10  2004/07/14 22:11:01  l2chef
 * Hp/Mp increase added (NightMarez)
 *
 * Revision 1.9  2004/07/12 20:57:21  l2chef
 * warehouses added (nuocnan)
 * pickup item is broadcasted
 *
 * Revision 1.8  2004/07/11 22:52:31  l2chef
 * if players connection is down, the connection is NULL
 * just ignore this
 *
 * Revision 1.7  2004/07/11 11:38:32  l2chef
 * crash prevention... dont try to pickup non-items
 *
 * Revision 1.6  2004/07/07 23:51:54  l2chef
 * pAtk update contributed by MetalRabit
 *
 * Revision 1.5  2004/07/04 17:40:28  l2chef
 * addexp and addsp combined to one ... they will never appear separated... i think
 *
 * Revision 1.4  2004/07/04 14:34:24  jeichhorn
 * added method to add exp and sp
 *
 * Revision 1.3  2004/07/04 11:13:27  l2chef
 * logging is used instead of system.out
 *
 * Revision 1.2  2004/06/27 08:51:42  jeichhorn
 * Added copyright notice
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.gameserver.model;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;
import java.util.logging.Logger;

import net.sf.l2j.gameserver.ClientThread;
import net.sf.l2j.gameserver.Connection;
import net.sf.l2j.gameserver.ExperienceTable;
import net.sf.l2j.gameserver.LevelUpData;
import net.sf.l2j.gameserver.Modifiers;
import net.sf.l2j.gameserver.SkillTable;
import net.sf.l2j.gameserver.clientpackets.RequestWithDrawalParty;
import net.sf.l2j.gameserver.serverpackets.ActionFailed;
import net.sf.l2j.gameserver.serverpackets.CharInfo;
import net.sf.l2j.gameserver.serverpackets.CharMoveToLocation;
import net.sf.l2j.gameserver.serverpackets.DeleteObject;
import net.sf.l2j.gameserver.serverpackets.GetItem;
import net.sf.l2j.gameserver.serverpackets.InventoryUpdate;
import net.sf.l2j.gameserver.serverpackets.MoveToPawn;
import net.sf.l2j.gameserver.serverpackets.MyTargetSelected;
import net.sf.l2j.gameserver.serverpackets.NpcInfo;
import net.sf.l2j.gameserver.serverpackets.PartySmallWindowUpdate;
import net.sf.l2j.gameserver.serverpackets.ServerBasePacket;
import net.sf.l2j.gameserver.serverpackets.SpawnItem;
import net.sf.l2j.gameserver.serverpackets.StatusUpdate;
import net.sf.l2j.gameserver.serverpackets.StopMove;
import net.sf.l2j.gameserver.serverpackets.SystemMessage;
import net.sf.l2j.gameserver.serverpackets.UserInfo;
import net.sf.l2j.gameserver.templates.L2Armor;
import net.sf.l2j.gameserver.templates.L2Weapon;

/**
 * This class represents a player character in the world.
 * there is always a client-thread connected to this 
 * (except if a player-store is activated upon logout)
 * 
 * @version $Revision: 1.3 $ $Date: 2004/08/05 00:53:40 $
 */
public class L2PcInstance extends L2Character
{
	private static Logger _log = Logger.getLogger(L2PcInstance.class.getName());
	private Connection _netConnection;

	private int _charId = 0x00030b7a;
	private int _canCraft = 0;
	private int _exp;
	private int _sp;
	private int _karma;
	private int _pvpKills;
	private int _pkKills;
	private int _pvpFlag;
	private int _clanId;
	private int _maxLoad;
	private int _race;
	private int _classId;
	private int _deleteTimer;
	private Inventory _inventory = new Inventory();
	private Warehouse _warehouse = new Warehouse();
	private int _moveType = 1;
	private int _waitType = 1;
	private int _crestId;
	private Map _skills = new HashMap();
	private Map _quests = new HashMap();
	private Map _shortCuts = new TreeMap();
	private int _allyId;

	//	pet
	private L2PetInstance _pet = null;
	
	// these values are only stored temporarily
	private ArrayList _partyMembers;
	private boolean _partyMatchingAutomaticRegistration;
	private boolean _partyMatchingShowLevel;
	private boolean _partyMatchingShowClass;
	private String _partyMatchingMemo;
	
	// this is needed for mobs that use moveToPawn
	private ArrayList _positionListeners = new ArrayList();
	// this reduces the number of updates for position listeners
	private int _validateCounter;

	// this is needed to find the inviting player for Party response
	// there can only be one active party request at once
	private L2PcInstance _currentPartyInviter;
	
	/**
	 * 
	 * @param newSkill
	 * @return L2Skill old skill that was upgraded or NULL if there was no lower skill of same type
	 */
	public L2Skill addSkill(L2Skill newSkill)
	{
		L2Skill oldSkill = (L2Skill) _skills.put(new Integer(newSkill.getId()), newSkill);
		return oldSkill;
	}
	

	/**
	 * 
	 * @param skill
	 * @return L2Skill old skill that was removed or NULL if the skill did not exist
	 */
	public L2Skill removeSkill(L2Skill skill)
	{
		return (L2Skill) _skills.remove(new Integer(skill.getId()));
	}
	
	/**
	 * @return L2Skill[]  all skills that this char knows
	 */
	public L2Skill[] getAllSkills()
	{
		return (L2Skill[]) _skills.values().toArray(new L2Skill[_skills.values().size()]);
	}
	
	public L2ShortCut[] getAllShortCuts()
	{
		return (L2ShortCut[]) _shortCuts.values().toArray(new L2ShortCut[_shortCuts.values().size()]);
	}
	
	public L2ShortCut getShortCut(int slot)
	{
		return (L2ShortCut) _shortCuts.get(new Integer(slot));
	}
	
	public void registerShortCut(L2ShortCut shortcut)
	{
		_shortCuts.put(new Integer(shortcut.getSlot()), shortcut);
	}

	/**
	 * @param slot
	 */
	public void deleteShortCut(int slot)
	{
		_shortCuts.remove(new Integer(slot));
	}
	
	
	/**
	 * Determine the level of a certain skill.
	 * this is used for shortcut reg
	 * @param skillId
	 * @return
	 */
	public int getSkillLevel(int skillId)
	{
		L2Skill skill = (L2Skill)_skills.get(new Integer(skillId));
		if (skill == null)
		{
			return -1;
		}
		else
		{
			return skill.getLevel();
		}
	}
	
	/**
	 * @param pvpFlag
	 */
	public void setPvpFlag(int pvpFlag)
	{
		_pvpFlag = pvpFlag;
	}

	/**
	 * @return
	 */
	public int getCanCraft()
	{
		return _canCraft;
	}

	/**
	 * @param canCraft
	 */
	public void setCanCraft(int canCraft)
	{
		_canCraft = canCraft;
	}

	/**
	 * @return
	 */
	public int getPkKills()
	{
		return _pkKills;
	}

	/**
	 * @param pkKills
	 */
	public void setPkKills(int pkKills)
	{
		_pkKills = pkKills;
	}

	/**
	 * @return
	 */
	public int getDeleteTimer()
	{
		return _deleteTimer;
	}

	/**
	 * @param deleteTimer
	 */
	public void setDeleteTimer(int deleteTimer)
	{
		_deleteTimer = deleteTimer;
	}

	/**
	 * @return
	 */
	public int getCurrentLoad()
	{
		return _inventory.getTotalWeight();
	}

	/**
	 * @return
	 */
	public int getKarma()
	{
		return _karma;
	}

	/**
	 * @param karma
	 */
	public void setKarma(int karma)
	{
		_karma = karma;
	}

	/**
	 * @return
	 */
	public int getMaxLoad()
	{
		return _maxLoad;
	}

	/**
	 * @param maxLoad
	 */
	public void setMaxLoad(int maxLoad)
	{
		_maxLoad = maxLoad;
	}

	/**
	 * @return
	 */
	public int getPvpKills()
	{
		return _pvpKills;
	}

	/**
	 * @param pvp
	 */
	public void setPvpKills(int pvpKills)
	{
		_pvpKills = pvpKills;
	}

	/**
	 * @return
	 */
	public int getClassId()
	{
		return _classId;
	}

	/**
	 * @param className
	 */
	public void setClassId(int classId)
	{
		_classId = classId;
	}

	/**
	 * @return
	 */
	public int getExp()
	{
		return _exp;
	}
	
	public void addExpAndSp(int addToExp, int addToSp)
	{
	    _log.fine("adding " + addToExp + " exp and " +addToSp+" sp to " + getName());
	    _exp += addToExp;
	    _sp += addToSp;
	    
		StatusUpdate su = new StatusUpdate(getObjectId());
		su.addAttribute(StatusUpdate.EXP, _exp);
		su.addAttribute(StatusUpdate.SP, _sp);
		sendPacket(su);
		
		SystemMessage sm = new SystemMessage(SystemMessage.YOU_EARNED_S1_EXP_AND_S2_SP);
		sm.addNumber(addToExp);
		sm.addNumber(addToSp);
		sendPacket(sm);
	    
	    if (_exp >= ExperienceTable.getInstance().getExp(getLevel() + 1))
	    {
	        // Looks like the level should be increased.
	        increaseLevel();
	        rewardSkills();
	    }
	}
	
	/**
	 *  reward player with free skill at certain lvl's.
	 */
	private void rewardSkills()
	{
		int charclass = getClassId();
		int lvl = getLevel();
		if (lvl == 5) // remove the noobie skills
		{		
			L2Skill skill = SkillTable.getInstance().getInfo(194, 1);
			removeSkill(skill);
			_log.fine("removed skill 'Lucky' from "+getName());
		}
		else if (lvl == 20) // give lvl 20 free skills
		{
			L2Skill skill = SkillTable.getInstance().getInfo(239, 1);
			addSkill(skill);
			_log.fine("awarded "+getName()+" with expertise D.");
		}
		else if (lvl == 40) // give lvl 40 free skills
		{
			L2Skill skill = SkillTable.getInstance().getInfo(239, 2);
			addSkill(skill);
			_log.fine("awarded "+getName()+" with expertise C.");
		}
		else if (lvl == 52) // give lvl 52 free skills
		{
			L2Skill skill = SkillTable.getInstance().getInfo(239, 3);
			addSkill(skill);
			_log.fine("awarded "+getName()+" with expertise B.");
		}		
		else
		{
			_log.fine("No skills awarded at lvl: "+lvl);
		}
	}
	

	/**
	 * @param exp
	 */
	public void setExp(int exp)
	{
		_exp = exp;
	}

	/**
	 * @return
	 */
	public int getRace()
	{
		return _race;
	}

	/**
	 * @param race
	 */
	public void setRace(int race)
	{
		_race = race;
	}

	/**
	 * @return
	 */
	public int getSp()
	{
		return _sp;
	}
	
	/**
	 * @param sp
	 */
	public void setSp(int sp)
	{
		_sp = sp;
	}

	/**
	 * @return
	 */
	public int getPvpFlag()
	{
		return _pvpFlag;
	}

	/**
	 * @return
	 */
	public int getClanId()
	{
		return _clanId;
	}

	/**
	 * @param clanId
	 */
	public void setClanId(int clanId)
	{
		_clanId = clanId;
	}

	/**
	 * @return
	 */
	public Inventory getInventory()
	{
		return _inventory;
	}

	/**
	 * @return
	 */
	public int getMoveType()
	{
		return _moveType;
	}

	/**
	 * @param moveType The moveType to set.
	 */
	public void setMoveType(int moveType)
	{
		_moveType = moveType;
		setRunning(_moveType == 1);
	}

	/**
	 * @return
	 */
	public int getWaitType()
	{
		return _waitType;
	}

	/**
	 * @param waitType The waitType to set.
	 */
	public void setWaitType(int waitType)
	{
		_waitType = waitType;
	}

	public Warehouse getWarehouse()
	{
		return _warehouse;
	}
	
	/**
	 * @return Returns the charId.
	 */
	public int getCharId()
	{
		return _charId;
	}

	/**
	 * @param charId The charId to set.
	 */
	public void setCharId(int charId)
	{
		_charId = charId;
	}

	/**
	 * @return
	 */
	public int getAdena()
	{
		return _inventory.getAdena();
	}

	/**
	 * @param adena The adena to set.
	 */
	public void reduceAdena(int adena)
	{
		_inventory.reduceAdena(adena);
	}

	/**
	 * @param adena The adena to set.
	 */
	public void addAdena(int adena)
	{
		_inventory.addAdena(adena);
	}

	/**
	 * @return
	 */
	public Connection getNetConnection()
	{
		return _netConnection;
	}

	/**
	 * @param connection
	 */
	public void setNetConnection(Connection connection)
	{
		_netConnection = connection;
	}

	/**
	 * @return
	 */
	public int getCrestId()
	{
		return _crestId;
	}

	/**
	 * @param crestId The crestId to set.
	 */
	public void setCrestId(int crestId)
	{
		_crestId = crestId;
	}
	
	public void onAction(L2PcInstance player)
	{
		if (player.getTarget() != this)
		{
			player.setTarget(this);
			MyTargetSelected my = new MyTargetSelected(getObjectId(), 0);
			player.sendPacket(my);
		}
		else
		{
			player.moveTo(getX(), getY(), getZ(), 36);
		}
	}
	
	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Character#broadCastUpdate()
	 */
	public void broadcastStatusUpdate()
	{
		StatusUpdate su = new StatusUpdate(getObjectId());
		su.addAttribute(StatusUpdate.CUR_HP, (int) getCurrentHp());
		su.addAttribute(StatusUpdate.CUR_MP, (int) getCurrentMp());

		super.broadcastStatusUpdate();
		
		if (getNetConnection() != null)
		{
			sendPacket(su);
		}
		
		ArrayList members = getPartyMembers();
		if (members != null)
		{
			PartySmallWindowUpdate update = new PartySmallWindowUpdate(this);
			
			for (int i = 0; i < members.size(); i++)
			{
				L2PcInstance temp = (L2PcInstance) members.get(i);
				temp.sendPacket(update);
			}
		}
	}

	public void notifyPositionListeners(boolean force)
	{
		ArrayList list = getPositionListeners();
		if (list.isEmpty())
		{
			return;
		}
		
		_validateCounter++;
		
		if (force || _validateCounter > 3)
		{
			int x = getX();
			int y = getY();
			int z = getZ();
	
			_log.fine("notifing "+list.size()+" followers");
			for (int i=0; i<list.size(); i++)
			{
				L2Character temp = (L2Character) list.get(i);
				temp.updateMoveTo(x,y,z);
			}
			
			_validateCounter = 0;
		}
	}
	
	/**
	 * @return
	 */
	public int getAllyId()
	{
		return _allyId;
	}
	/**
	 * @param allyId The allyId to set.
	 */
	public void setAllyId(int allyId)
	{
		_allyId = allyId;
	}
	
	protected void onHitTimer(L2Character target, int damage, boolean crit, boolean miss, boolean soulshot)
	{
		if (crit)
		{
			sendPacket(new SystemMessage(SystemMessage.CRITICAL_HIT));
		}
		
		if (miss)
		{
			sendPacket(new SystemMessage(SystemMessage.MISSED_TARGET));
		}
		else
		{
			SystemMessage sm = new SystemMessage(SystemMessage.YOU_DID_S1_DMG);
			sm.addNumber(damage);
			sendPacket(sm);
		}

		super.onHitTimer(target, damage, crit, miss, soulshot);
	}
	
	public void sendPacket(ServerBasePacket packet)
	{
		try
		{
			getNetConnection().sendPacket(packet);
		}
		catch (Exception e)
		{
//			e.printStackTrace();
			// the connection for this player seems to be broken. linkdead ? logout ?
			// TODO automatically remove all references to this player
		}
	}
	
	/**
	 * 
	 */
	protected void startCombat()
	{
		L2Character target = (L2Character) getTarget();
		if (target == null)
		{
			_log.warning("failed to start combat without target.");
			sendPacket(new ActionFailed());
		}
		else if (getAttackRange() <= getDistance(target.getX(),target.getY()))
		{
			sendPacket(new ActionFailed());
		}
		else
		{
			_log.fine("starting combat");
			super.startCombat();
		}	
		
	}
	
	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Character#onTargetReached()
	 */
	public void onTargetReached()
	{
		notifyPositionListeners(true);
		
		super.onTargetReached();
		
		try
		{
			switch (getCurrentState())
			{
				case STATE_PICKUP_ITEM:
					doPickupItem();
					break;
				case STATE_ATTACKING:
					startCombat();
					break;
				default:
					break;
			}
		}
		catch (IOException io)
		{
			io.printStackTrace();
		}
	}
	
	private void doPickupItem() throws IOException
	{
		setCurrentState(STATE_IDLE);

		if (! (getTarget() instanceof L2ItemInstance))
		{
			// dont try to pickup anything that is not an item :)
			_log.warning("trying to pickup wrong target."+getTarget());
			return;
		}
		
		L2ItemInstance target = (L2ItemInstance) getTarget();
		
		sendPacket(new ActionFailed());
		
		StopMove sm = new StopMove(getObjectId(), getX(), getY(), getZ(), getHeading()); 
		_log.fine("pickup pos: "+ target.getX() + " "+target.getY()+ " "+target.getZ() );
		sendPacket(sm);
		
		boolean pickupOk = false;
		
		synchronized (target)
		{
			if (target.isOnTheGround())
			{
				pickupOk = true;
				target.setOnTheGround(false);
			}
		}
		
		if (!pickupOk)
		{
			// someone else was faster
			return;
		}
		
		L2World.getInstance().removeVisibleObject(target);

		GetItem gi = new GetItem(target, getObjectId());
		sendPacket(gi);
		broadcastPacket(gi);

	
		if (target.getItemId() == 57)
		{
			SystemMessage smsg = new SystemMessage(28); // you picked up $s1 adena
			smsg.addNumber(target.getCount());
			sendPacket(smsg);
		}
		else if (target.getCount() == 1)
		{
			SystemMessage smsg = new SystemMessage(30); // you picked up $s1
			smsg.addItemName(target.getItemId());
			sendPacket(smsg);
		}
		else
		{
			SystemMessage smsg = new SystemMessage(29); // you picked up $s1$s2
			smsg.addNumber(target.getCount());
			smsg.addItemName(target.getItemId());
			sendPacket(smsg);
		}
	
		L2ItemInstance target2 = getInventory().addItem(target);
		InventoryUpdate iu = new InventoryUpdate();
		if (target2.getLastChange() == L2ItemInstance.ADDED)
		{
			iu.addNewItem(target);
		}
		else
		{
			iu.addModifiedItem(target2);
		}
		
		sendPacket(iu);
	
		//TODO send only weight update, if possible
		UserInfo ci = new UserInfo(this);
		sendPacket(ci);
	}
	
	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Character#setTarget(net.sf.l2j.gameserver.model.L2Object)
	 */
	public void setTarget(L2Object newTarget)
	{
		// cancel old target
		L2Object oldTarget = getTarget();
		
		if (oldTarget != null)
		{
			if (oldTarget.equals(newTarget))
			{
				return; // no target change
			}
		
			// remove listener if the target was a char
			if (oldTarget instanceof L2Character)
			{
				((L2Character) oldTarget).removeStatusListener(this);
			}
		}

		if (newTarget != null && newTarget instanceof L2Character)
		{
			((L2Character) newTarget).addStatusListener(this);
		}
		
		super.setTarget(newTarget);
	}


	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Character#getActiveWeapon()
	 */
	public L2Weapon getActiveWeapon()
	{
		L2ItemInstance weapon = getInventory().getPaperdollItem(Inventory.PAPERDOLL_RHAND);
		if (weapon == null)
		{
			return null;
		}
		return (L2Weapon) weapon.getItem();
	}
	
	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Character#reduceCurrentHp(int, net.sf.l2j.gameserver.model.L2Character)
	 */
	public void reduceCurrentHp(int i, L2Character attacker) 
	{
		super.reduceCurrentHp(i, attacker);
		if (isDead() && getPet() != null)
		{
			getPet().deleteMe(this);
		}
		
		if (attacker != null)
		{
			SystemMessage smsg = new SystemMessage(SystemMessage.S1_GAVE_YOU_S2_DMG);
			_log.fine("Attacker:"+attacker.getName());
			if (attacker instanceof L2MonsterInstance || attacker instanceof L2NpcInstance)
			{
				int mobId = ((L2NpcInstance) attacker).getNpcTemplate().getNpcId();
				_log.fine("mob id:"+mobId);
				smsg.addNpcName(mobId);
			}
			else
			{
				smsg.addString( attacker.getName() );
			}
			
			smsg.addNumber(i);
			sendPacket(smsg);
		}
	}


	
	/**
	 * @param b
	 */
	public void setPartyMatchingAutomaticRegistration(boolean b)
	{
		_partyMatchingAutomaticRegistration = b;
	}


	/**
	 * @param b
	 */
	public void setPartyMatchingShowLevel(boolean b)
	{
		_partyMatchingShowLevel = b;
	}


	/**
	 * @param b
	 */
	public void setPartyMatchingShowClass(boolean b)
	{
		_partyMatchingShowClass = b;
	}


	/**
	 * @param memo
	 */
	public void setPartyMatchingMemo(String memo)
	{
		_partyMatchingMemo = memo;
	}
	
	public boolean isPartyMatchingAutomaticRegistration()
	{
		return _partyMatchingAutomaticRegistration;
	}
	
	public String getPartyMatchingMemo()
	{
		return _partyMatchingMemo;
	}
	
	public boolean isPartyMatchingShowClass()
	{
		return _partyMatchingShowClass;
	}
	
	public boolean isPartyMatchingShowLevel()
	{
		return _partyMatchingShowLevel;
	}


	/**
	 * @param client
	 * @return
	 */
	public boolean setPendingJoinPartyRequest(L2PcInstance requestor)
	{
		if (_currentPartyInviter == null)
		{
			_currentPartyInviter = requestor;
			return true;
		}
		else
		{
			return false;
		}
	}


	/**
	 * @return
	 */
	public L2PcInstance getPartyInviter()
	{
		L2PcInstance result = _currentPartyInviter;
		_currentPartyInviter = null;
		return result;
	}

	/**
	 * this method is called to make an object known to a player without
	 * triggering a spawn packet. its required for item drops, since the
	 * client already registers items when it receives the drop packet.
	 * an additional spawn packet causes duplicate items
	 * @param object
	 */
	public void addKnownObjectWithoutCreate(L2Object object)
	{
		super.addKnownObject(object);
	}
	
	
	public void addKnownObject(L2Object object)
	{
		super.addKnownObject(object);

		if (object instanceof L2ItemInstance)
		{
			SpawnItem si = new SpawnItem((L2ItemInstance) object);
			sendPacket(si);
		}
		else if (object instanceof L2NpcInstance)
		{
			NpcInfo ni = new NpcInfo((L2NpcInstance) object);
			sendPacket(ni);
		}
		else if (object instanceof L2PetInstance)
		{
			NpcInfo ni = new NpcInfo((L2PetInstance) object);
			sendPacket(ni);
		}
		else if (object instanceof L2PcInstance)
		{
			// send player info to our client
			L2PcInstance otherPlayer = (L2PcInstance) object;
			sendPacket(new CharInfo(otherPlayer));
		}
		
		if (object instanceof L2Character)
		{
			L2Character obj = (L2Character) object;
			if (obj.isMoving()) 
			{
				_log.fine("Spotted object in movement, updating status");
				CharMoveToLocation mov = new CharMoveToLocation(obj);
				sendPacket(mov);
			} 
			else if (obj.isMovingToPawn())
			{
				_log.fine("Spotted object in movement to pawn, updating status");
				MoveToPawn mov = new MoveToPawn(obj, obj.getPawnTarget(), obj.getPawnOffset());
				sendPacket(mov);					
			}
		}
		
	}
	
	public void removeKnownObject(L2Object object)
	{
		super.removeKnownObject(object);
		sendPacket(new DeleteObject(object));
	}
	
	
	public void increaseLevel()
	{
		super.increaseLevel();
		
		L2LvlupData lvlData = LevelUpData.getInstance().getTemplate(getClassId());
		
		// the new HP and MP max values
		if (lvlData != null)
		{
			if (getLevel() == 2)
			{
				int newhp = (int) Math.rint(lvlData.getDefaulthp()+lvlData.getDefaulthpadd());
				int newmp = (int) Math.rint(lvlData.getDefaultmp()+lvlData.getDefaultmpadd());
				// increase hp
				setMaxHp(newhp);
				setCurrentHp(newhp);
				StatusUpdate suhpmax = new StatusUpdate(getObjectId());
				suhpmax.addAttribute(StatusUpdate.MAX_HP, newhp);
				sendPacket(suhpmax);
				StatusUpdate suhp = new StatusUpdate(getObjectId());
				suhp.addAttribute(StatusUpdate.CUR_HP, newhp);
				sendPacket(suhp);
				// increase mp
				setMaxMp(newmp);
				setCurrentMp(newmp);
				StatusUpdate sumpmax = new StatusUpdate(getObjectId());
				sumpmax.addAttribute(StatusUpdate.MAX_MP, newmp);
				sendPacket(sumpmax);
				StatusUpdate sump = new StatusUpdate(getObjectId());
				sump.addAttribute(StatusUpdate.CUR_MP, newmp);
				sendPacket(sump);
			} 
			else 
			{
				double hp1 = (getLevel() - 1) * lvlData.getDefaulthpadd();
				double hp2 = (getLevel() - 2) * lvlData.getDefaulthpbonus();
				double mp1 = (getLevel() - 2) * lvlData.getDefaultmpadd();
				double mp2 = (getLevel() - 2) * lvlData.getDefaultmpadd();
				int newhp = (int) Math.rint(lvlData.getDefaulthp()+hp1+hp2);
				int newmp = (int) Math.rint(lvlData.getDefaultmp()+mp1+mp2);
				// increase hp
				setMaxHp(newhp);
				setCurrentHp(newhp);
				StatusUpdate su = new StatusUpdate(getObjectId());
				su.addAttribute(StatusUpdate.MAX_HP, newhp);
				su.addAttribute(StatusUpdate.CUR_HP, newhp);
				// increase mp
				setMaxMp(newmp);
				setCurrentMp(newmp);
				su.addAttribute(StatusUpdate.MAX_MP, newmp);
				su.addAttribute(StatusUpdate.CUR_MP, newmp);
				sendPacket(su);
			}
		}
		else
		{
			_log.warning("No lvl up data for class id: "+getClassId());
		}						
	}
	


	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.model.L2Character#getAtkRange()
	 */
	public int getAttackRange()
	{
		L2Weapon weapon = getActiveWeapon();
		if (weapon != null && weapon.getWeaponType() == L2Weapon.WEAPON_TYPE_BOW)
		{
			return 500; //bow range
		}

		return 36;// meelee range
	}
	
	public ArrayList getPartyMembers()
	{
		return _partyMembers;
	}

	public void setPartyMembers(ArrayList party)
	{
		_partyMembers = party;
	}
	
	public ArrayList getPositionListeners()
	{
		return _positionListeners;
	}
	
	public void addPositionListener(L2Character follower)
	{
		_positionListeners.add(follower);
	}
	
	public void removePositionListener(L2Character follower)
	{
		_positionListeners.remove(follower);
	}
	
	public void stopAllTimers()
	{
		stopAttackTask();
		stopHitTask();
		stopMpRegeneration();
		stopHpRegeneration();
		stopMove();
	}
	
	public L2PetInstance getPet()
	{
		return _pet;
	}

	public void setPet(L2PetInstance pet)
	{
		_pet = pet;
	}
	
	public void deleteMe(ClientThread client)
	{
		setTarget(null);
		L2World world = L2World.getInstance();
		world.removeVisibleObject(this);
		
		if (getPartyMembers() != null && getPartyMembers().size() > 0)
		{
			RequestWithDrawalParty leaveparty = new RequestWithDrawalParty(null, client);
		}
		
		if (getPet() != null)
		{
			getPet().deleteMe(this);// removes pet from game
		}
		removeAllKnownObjects();
		setNetConnection(null);
		world.removeObject(this);
	}

	
	/**
	 * 
	 */
	public void updatePAtk()
	{
		double wpnDam = 3.0;

		L2Weapon weapon = getActiveWeapon();
		L2Modifiers modifier = Modifiers.getInstance().getTemplate(getClassId());
		if (weapon != null && modifier != null)
		{
			double weapondmg = weapon.getPDamage();
			double lvlmod = (100.0 - 11 +getLevel()) / 100.0;
			double strmod = (100.0 + modifier.getModstr()) / 100.0;
		
			double pAtk = weapondmg * lvlmod * strmod;
			setPhysicalAttack((int) Math.rint(pAtk));
		
			_log.fine("new patk: "+pAtk + " weapon patk: "+weapondmg);
		}
		else
		{
			double weapondmg = wpnDam;
			double lvlmod = (100.0 - 11 +getLevel()) / 100.0;
			double strmod = (100.0 + modifier.getModstr()) / 100.0;
			
			double pAtk = weapondmg * lvlmod * strmod;
			setPhysicalAttack((int) Math.rint(pAtk));
			
			_log.fine("new patk: "+pAtk + " weapon patk: "+weapondmg);			
		}
	}
	
	public void updatePDef()
	{
		double pDef = 0; // bonus(skills)+ (total armor pdef) * (level bonus)
		
		double lvlBonus = ((getLevel()-11)/100) + 1.00;
		L2Armor armorPiece;
		L2ItemInstance dummy;
		
		double totalItemDef = 80.00 * lvlBonus; //TODO: default pDef depending on class
		
		dummy = getInventory().getPaperdollItem(Inventory.PAPERDOLL_BACK);
		if (dummy != null)
		{
			armorPiece = (L2Armor) dummy.getItem();
			totalItemDef += armorPiece.getPDef();
		}

		dummy = getInventory().getPaperdollItem(Inventory.PAPERDOLL_CHEST);
		if (dummy != null)
		{
			armorPiece = (L2Armor) dummy.getItem();
			totalItemDef += armorPiece.getPDef();
		}

		dummy = getInventory().getPaperdollItem(Inventory.PAPERDOLL_FEET);
		if (dummy != null)
		{
			armorPiece = (L2Armor) dummy.getItem();
			totalItemDef += armorPiece.getPDef();
		}

		dummy = getInventory().getPaperdollItem(Inventory.PAPERDOLL_GLOVES);
		if (dummy != null)
		{
			armorPiece = (L2Armor) dummy.getItem();
			totalItemDef += armorPiece.getPDef();
		}
		
		dummy = getInventory().getPaperdollItem(Inventory.PAPERDOLL_HEAD);
		if (dummy != null)
		{
			armorPiece = (L2Armor) dummy.getItem();
			totalItemDef += armorPiece.getPDef();
		}

		dummy = getInventory().getPaperdollItem(Inventory.PAPERDOLL_LEGS);
		if (dummy != null)
		{
			armorPiece = (L2Armor) dummy.getItem();
			totalItemDef += armorPiece.getPDef();
		}
		dummy = getInventory().getPaperdollItem(Inventory.PAPERDOLL_UNDER);
		if (dummy != null)
		{
			armorPiece = (L2Armor) dummy.getItem();
			totalItemDef += armorPiece.getPDef();
		}

		pDef = totalItemDef * lvlBonus;
		
		setPhysicalDefense((int) Math.round(pDef)); 
		_log.fine(getObjectId()+": new pdef: "+pDef);
	}

	public void updateMAtk()
	{
		double wpnDam = 3.0;
		
		L2Weapon weapon = getActiveWeapon();
		L2Modifiers modifier = Modifiers.getInstance().getTemplate(getClassId());
		if (weapon != null && modifier != null)
		{
			double weapondmg = weapon.getMDamage();
			double lmod = (100.0 - 11 +getLevel()) / 100.0;
			double lvlmod = Math.sqrt(lmod);
			double imod = (100.0 + modifier.getModint()) / 100.0;
			double intmod = Math.sqrt(imod);
			
			double mAtk = weapondmg * lvlmod * intmod; 
			setMagicalAttack((int) Math.rint(mAtk));
			_log.fine("new matk: "+mAtk + " weapon matk: "+weapondmg);
		}
		else
		{
			double weapondmg = wpnDam;
			double lmod = (100.0 - 11 +getLevel()) / 100.0;
			double lvlmod = Math.sqrt(lmod);
			double imod = (100.0 + modifier.getModint()) / 100.0;
			double intmod = Math.sqrt(imod);
			
			double mAtk = weapondmg * lvlmod * intmod; 
			setMagicalAttack((int) Math.rint(mAtk));
			_log.fine("new matk: "+mAtk + " weapon matk: "+weapondmg);
			
		}
	}
	
	public void updateMDef()
	{
		double mDef = 0; // (total armor mdef) * (level bonus) * (MEN bonus)
		
		double lvlBonus = ((getLevel()-11)/100) + 1;
		double MENbonus = 1.00; //TODO: MEN bonus depending on class 
		L2Armor armorPiece;
		L2ItemInstance dummy;
		
		double totalItemDef = 40 * lvlBonus; //TODO: default mDef depending on class
		
		dummy = getInventory().getPaperdollItem(Inventory.PAPERDOLL_LEAR);
		if (dummy != null)
		{
			armorPiece = (L2Armor) dummy.getItem();
			totalItemDef += armorPiece.getMDef();
		}

		dummy = getInventory().getPaperdollItem(Inventory.PAPERDOLL_LFINGER);
		if (dummy != null)
		{
			armorPiece = (L2Armor) dummy.getItem();
			totalItemDef += armorPiece.getMDef();
		}

		dummy = getInventory().getPaperdollItem(Inventory.PAPERDOLL_NECK);
		if (dummy != null)
		{
			armorPiece = (L2Armor) dummy.getItem();
			totalItemDef += armorPiece.getMDef();
		}

		dummy = getInventory().getPaperdollItem(Inventory.PAPERDOLL_REAR);
		if (dummy != null)
		{
			armorPiece = (L2Armor) dummy.getItem();
			totalItemDef += armorPiece.getMDef();
		}

		dummy = getInventory().getPaperdollItem(Inventory.PAPERDOLL_RFINGER);
		if (dummy != null)
		{
			armorPiece = (L2Armor) dummy.getItem();
			totalItemDef += armorPiece.getMDef();
		}

		mDef = totalItemDef * lvlBonus * MENbonus;
		
		setMagicalDefense((int) Math.round(mDef)); 
		_log.fine(getObjectId()+": new mdef: "+mDef);
	}
}

