
package net.sf.l2j.gameserver.model;
import net.sf.l2j.gameserver.serverpackets.PetStatusUpdate;

import java.io.IOException;
import java.util.logging.Logger;
import net.sf.l2j.gameserver.ExperienceTable;
import net.sf.l2j.gameserver.templates.L2Npc;
import net.sf.l2j.gameserver.templates.L2Weapon;
import net.sf.l2j.gameserver.serverpackets.ChangeMoveType;
import net.sf.l2j.gameserver.serverpackets.DeleteObject;
import net.sf.l2j.gameserver.serverpackets.MyTargetSelected;
import net.sf.l2j.gameserver.serverpackets.PetInventoryUpdate;
import net.sf.l2j.gameserver.serverpackets.PetStatusShow;
import net.sf.l2j.gameserver.serverpackets.ActionFailed;
import net.sf.l2j.gameserver.serverpackets.StatusUpdate;
import net.sf.l2j.gameserver.serverpackets.StopMove;
import net.sf.l2j.gameserver.serverpackets.SystemMessage;
import net.sf.l2j.gameserver.serverpackets.PetDelete
;




public class L2PetInstance extends L2Character{
	private static Logger _log = Logger.getLogger(L2PetInstance.class.getName());
	
	private int  _npcId =1;
	private byte _petId =1;
	private int _exp=111122;
	private int _sp=32;
	private int _pvpKills;
	private int _pkKills;
	private byte _pvpFlag;
	private int _maxFed=5;
	private int _curFed=1;
	private int _OwnerId=-1;
	private int _karma = 0;
	private Inventory _inventory = new Inventory();
	private L2Weapon _dummyWeapon; 
	private int _attackRange = 36;
	
	public L2PetInstance(L2Npc template)
	{
		
		setCollisionHeight(template.getHeight());
		setCollisionRadius(template.getRadius());
		setCurrentState(STATE_IDLE);
		setPhysicalAttack(9999);
	}
	
	
	public void onAction(L2PcInstance player)
	{
		if (player.getObjectId() == _OwnerId)
		{
			player.sendPacket(new PetStatusShow(2));
			player.sendPacket(new ActionFailed());
		}
		player.setCurrentState(L2Character.STATE_IDLE);
		_log.fine("new target selected:"+getObjectId());
		player.setTarget(this);
		MyTargetSelected my = new MyTargetSelected(getObjectId(), player.getLevel() - getLevel());
		player.sendPacket(my);
		
	}
	
	
	public int getPetId()
	{
		return _petId;
	}
	public void setPetId(byte petid)
	{
		_petId = petid;
	}
	public int getKarma()
	{
		return _karma;
	}
	public void setKarma(int karma)
	{
		_karma = karma;
	}
	public int getMaxFed()
	{
		return _maxFed;
	}
	public void setMaxFed(int num)
	{
		_maxFed = num;
	}
	public int getCurrentFed()
	{
		return _curFed;
	}
	public void setCurrentFed(int num)
	{
		_curFed = num;
	}
	public void setOwner(int Id)
	{
		_OwnerId = Id;
	}
	public int getOwner()
	{
		return _OwnerId;
	}
	public void setNpcId(int Id)
	{
		_npcId = Id;
	}
	public int getNpcId()
	{
		return _npcId;
	}
	public void setPvpFlag(byte pvpFlag)
	{
		_pvpFlag = pvpFlag;
	}
	public void setPkKills(int pkKills)
	{
		_pkKills = pkKills;
	}
	public int getPkKills()
	{
		return _pkKills;
	}
	public int getExp()
	{
		return _exp;
	}
	public void setExp(int exp)
	{
		_exp = exp;
	}
	public int getSp()
	{
		return _sp;
	}
	
	public void setSp(int sp)
	{
		_sp = sp;
	}
	public void addExpAndSp(int addToExp, int addToSp)
	{
		_log.fine("adding " + addToExp + " exp and " +addToSp+" sp to " + getName());
		_exp += addToExp;
		_sp += addToSp;
		
		StatusUpdate su = new StatusUpdate(getObjectId());
		su.addAttribute(StatusUpdate.EXP, _exp);
		su.addAttribute(StatusUpdate.SP, _sp);
		sendPacket(su);
		
		SystemMessage sm = new SystemMessage(SystemMessage.YOU_EARNED_S1_EXP_AND_S2_SP);
		sm.addNumber(addToExp);
		sm.addNumber(addToSp);
		sendPacket(sm);
		
		if (_exp >= ExperienceTable.getInstance().getExp(getLevel() + 1))
		{
			// Looks like the level should be increased.
			increaseLevel();
		}
	}
	public void followOwner(L2PcInstance owner)
	{
		setCurrentState(STATE_FOLLOW);
		setTarget(owner);
		moveTo(owner.getX(),owner.getY(),owner.getZ(), 30);
		broadcastPacket(new PetStatusUpdate(this,false));
	}
	public void onTargetReached()
	{	
		super.onTargetReached();
		try
		{
			switch (getCurrentState())
			{
			case STATE_PICKUP_ITEM:
				doPickupItem();
				break;
			case STATE_ATTACKING:
				startCombat();
				break;
			default:
				break;
			}
		}
		catch (IOException io)
		{
			io.printStackTrace();
		}
	}
	
	public L2Weapon getActiveWeapon()
	{
		return _dummyWeapon;
	}
	public void setPhysicalAttack(int physicalAttack)
	{
		super.setPhysicalAttack(physicalAttack);
		
		// create a dummy weapon 
		_dummyWeapon = new L2Weapon();
		_dummyWeapon.setPDamage(physicalAttack);
		int randDmg = getLevel();
		_dummyWeapon.setRandomDamage(getLevel());
		
	}
	public Inventory getInventory()
	{
		return _inventory;
	}
	private void doPickupItem() throws IOException
	{
		setCurrentState(STATE_IDLE);
		
		L2ItemInstance target =(L2ItemInstance)getTarget();
		if (! (getTarget() instanceof L2ItemInstance))
		{
			// dont try to pickup anything that is not an item :)
			_log.warning("trying to pickup wrong target."+getTarget());
			return;
		}
		
		L2World world = L2World.getInstance();
		L2PcInstance owner = (L2PcInstance)world.findObject(getOwner());
		
		getInventory().addItem(target);
		owner.sendPacket(new ActionFailed());
		
		StopMove sm = new StopMove(getObjectId(), getX(), getY(), getZ(), getHeading()); 
		_log.fine("pickup pos: "+ target.getX() + " "+target.getY()+ " "+target.getZ() );
		sendPacket(sm);
		
		DeleteObject dobj = new DeleteObject(target);
		owner.sendPacket(dobj);
		owner.broadcastPacket(dobj);
		
		
		
		L2World.getInstance().removeVisibleObject(target);
		// should also be deleted from client-threads known object list
		
		
		
		
		PetInventoryUpdate iu = new PetInventoryUpdate(getInventory().getItems());
		owner.sendPacket(iu);
		
		
	}
	public void reduceCurrentHp(int damage, L2Character attacker)
	{
		super.reduceCurrentHp(damage, attacker);
		// this is usually only called in combat
		
	
		if (!isDead() && attacker != null)
		{
			if (!isInCombat())
			{
				startAttack(attacker);
			}
			else
			{
				_log.fine("already attacking");
			}
		}
		if (isDead())
		{
			L2World world = L2World.getInstance();
			L2PcInstance owner = (L2PcInstance)world.findObject(getOwner());
			deleteMe(owner);
		}
		
	}
	public void broadcastStatusUpdate()
	{
		super.broadcastStatusUpdate();
		if (getOwner() != -1)
		{
			L2World world = L2World.getInstance();
			L2PcInstance owner = (L2PcInstance)world.findObject(getOwner());
			owner.sendPacket(new PetStatusUpdate(this,false));
		}
	}
	public void deleteMe(L2PcInstance owner)
	{
	
		L2World.getInstance().removeVisibleObject(this);
		removeAllKnownObjects();
		owner.setPet(null);
		setTarget(null);
		owner.sendPacket(new PetDelete(getObjectId(), 2));
	}
	public void startAttack(L2Character target)
	{
		if (!isRunning())
		{
			setRunning(true);
			ChangeMoveType move = new ChangeMoveType(this, ChangeMoveType.RUN);
			broadcastPacket(move);
		}
		
		super.startAttack(target);
	}
	public int getAttackRange()
	{
		return _attackRange; 
	}
	
	public void setAttackRange(int range)
	{
		_attackRange = range;
	}
	

}

