/*
 * $Header: /var/lib/gforge/chroot/cvsroot/nextg/CVSROOT/L2_Gameserver/java/net/sf/l2j/gameserver/model/L2World.java,v 1.3 2004/08/05 00:53:40 nextg Exp $
 *
 * $Author: nextg $
 * $Date: 2004/08/05 00:53:40 $
 * $Revision: 1.3 $
 * $Log: L2World.java,v $
 * Revision 1.3  2004/08/05 00:53:40  nextg
 * RuSh
 *
 * Revision 1.12  2004/08/02 00:08:32  l2chef
 * better solution to drop item problem
 *
 * Revision 1.11  2004/07/28 23:57:24  l2chef
 * loglevels changed
 * client crash fixed when picking up items twice
 *
 * Revision 1.10  2004/07/23 01:51:47  l2chef
 * all object spawn and delete is now handeld in L2PcInstance
 *
 * Revision 1.9  2004/07/18 17:39:35  l2chef
 * aggressive monsters scan for targets in range (FTPW)
 *
 * Revision 1.8  2004/07/17 23:12:00  l2chef
 * attack range is set for monsters
 *
 * Revision 1.7  2004/07/13 22:59:27  l2chef
 * world init removed
 *
 * Revision 1.6  2004/07/11 11:48:54  l2chef
 * additional npc data is used when populating the world
 *
 * Revision 1.5  2004/07/04 11:13:27  l2chef
 * logging is used instead of system.out
 *
 * Revision 1.4  2004/06/29 22:55:35  l2chef
 * tell and say works now. say is actually a server broadcast now
 *
 * Revision 1.3  2004/06/27 20:50:03  l2chef
 * better error message when data file is missing. skipping of comment lines
 *
 * Revision 1.2  2004/06/27 08:51:42  jeichhorn
 * Added copyright notice
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.gameserver.model;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.logging.Logger;

import net.sf.l2j.gameserver.IdFactory;
import net.sf.l2j.gameserver.NpcTable;
import net.sf.l2j.gameserver.templates.L2Npc;
import net.sf.l2j.gameserver.NpcDataTable;
import net.sf.l2j.gameserver.model.L2NpcData;

/**
 * This class ...
 * 
 * @version $Revision: 1.3 $ $Date: 2004/08/05 00:53:40 $
 */
public class L2World
{
	private static Logger _log = Logger.getLogger(L2World.class.getName());

	private HashMap _allPlayers;
	
    private HashMap _allObjects;
    
    private HashMap _visibleObjects;
	
	private IdFactory _idFactory;
	
	private static L2World _instance;

	private L2World()
	{
		_allPlayers = new HashMap();
		_allObjects = new HashMap();
		_visibleObjects = new HashMap();
		_idFactory = IdFactory.getInstance();
	}
	
	public static L2World getInstance()
	{
		if (_instance == null)
		{
			_instance = new L2World();
		}
		return _instance;
	}
	

	/**
	 * this fills the visible objects hashmap with the stored data.
	 * only items and NPCs will be restored.
	 * 
	 * @throws IOException
	 */
	public void initWorldContent() throws IOException
	{
		int items = 0;
		int npcs = 0;
		
		try
		{
			File worldData = new File("data/world.csv");
			LineNumberReader lnr = new LineNumberReader(new BufferedReader( new FileReader(worldData)));
			
			String line = null;
			while ( (line=lnr.readLine()) != null)
			{
				if (line.trim().length() == 0 || line.startsWith("#"))
				{
					//skip comments
					continue;
				}
				
				L2Object object = parseLine(line);
				if (object != null)	// skip comments and broken lines
				{
					if (object instanceof L2NpcInstance)
					{
						npcs++;
					}
					else if (object instanceof L2ItemInstance)
					{
						items++;
					}
					
					_visibleObjects.put(new Integer(object.getObjectId()), object);
					_allObjects.put(new Integer(object.getObjectId()), object);
				}
			}

			_log.config("restored " + items + " Items in the world");
			_log.config("restored " + npcs + " NPCs in the world");
		}
		catch (FileNotFoundException e)
		{
			_log.warning("world.csv is missing in data folder");
			throw new IOException("data/world.csv is missing.");
		}
		catch (Exception e)
		{
			e.printStackTrace();
			throw new IOException("data/world.csv could not be read." + e);
		}
	}
	
	private L2Object parseLine(String line)
	{
		StringTokenizer st = new StringTokenizer(line, ";");
		String type = st.nextToken();
		if (type.equals("1")) // NPC
		{
			String objId = st.nextToken();
			String npcTemplate = st.nextToken();
			L2Npc template = NpcTable.getInstance().getTemplate(Integer.parseInt(npcTemplate));
			if (template == null)
			{
				_log.warning("template id " + npcTemplate + " is missing !");
				return null;
			}

			String implementationName = st.nextToken(); // implementing class name
			Object[] parameters = {template};
			L2NpcInstance npc = null;
			
			try
			{
				npc = (L2NpcInstance) Class.forName("net.sf.l2j.gameserver.model." + implementationName + "Instance").getConstructors()[0].newInstance(parameters);
				
				int tempId = npc.getNpcTemplate().getNpcId();
				
				npc.setObjectId(Integer.parseInt(objId));
	
				String attackable = st.nextToken();
				npc.setAttackable(attackable.equals("1"));
	
				String x1 = st.nextToken();
				int x = Integer.parseInt(x1);
				npc.setX(x);
				npc.setXdestination(x);
				
				String y1 = st.nextToken();
				int y = Integer.parseInt(y1);
				npc.setY(y);
				npc.setYdestination(y);
				
				String z1 = st.nextToken();
				int z = Integer.parseInt(z1);
				npc.setZ(z);
				npc.setZdestination(z);
				
				String heading1 = st.nextToken();
				int heading = Integer.parseInt(heading1);
				npc.setHeading(heading);
				
				st.nextToken(); // always 0 ???
	
				String mSpd = st.nextToken(); 
				npc.setMagicalSpeed(Integer.parseInt(mSpd));
	
				String pSpd = st.nextToken(); 
				npc.setPhysicalSpeed(Integer.parseInt(pSpd));
				
				String runSpeed = st.nextToken(); 
				npc.setRunSpeed(Integer.parseInt(runSpeed));
				
				String walkSpeed = st.nextToken();
				npc.setWalkSpeed(Integer.parseInt(walkSpeed));
				st.nextToken(); // always 0 ???	speed in water ??
				st.nextToken(); // always 50 ??? speed in water ??
	
				String floatingRunSpeed = st.nextToken();
				npc.setFloatingRunSpeed(Integer.parseInt(floatingRunSpeed));
				
				String floatingWalkSpeed = st.nextToken();
				npc.setFloatingWalkSpeed(Integer.parseInt(floatingWalkSpeed));
				
				String flyingRunSpeed = st.nextToken();
				npc.setFlyingRunSpeed(Integer.parseInt(flyingRunSpeed));
				
				String flyingWalkSpeed = st.nextToken();
				npc.setFlyingWalkSpeed(Integer.parseInt(flyingWalkSpeed));
				
				String unknown1 = st.nextToken();
				npc.setUnknown1(Double.parseDouble(unknown1));	//seems to be fixed by template ?!
	
				String unknown2 = st.nextToken();
				npc.setUnknown2(Double.parseDouble(unknown2));	//seems to be fixed by template ?!
				
				st.nextToken(); // collision settings already in template
				st.nextToken(); // 
	
				String rightHand = st.nextToken(); //
				npc.setRightHandItem(Integer.parseInt(rightHand)); 
	
				st.nextToken(); // 
				String leftHand = st.nextToken(); // 
				npc.setLeftHandItem(Integer.parseInt(leftHand)); 
				
				st.nextToken(); //  always 1 ?!?
				String running = st.nextToken();
				npc.setRunning(running.equals("1"));
				
				st.nextToken(); // attacking = 1
				st.nextToken();	// dead = 1, alive = 0
				st.nextToken();
	
				if (tempId > 0)
				{
					L2NpcData npcID = NpcDataTable.getInstance().getTemplate(tempId);				
	
					if (npcID != null)
					{
						npc.setLevel(npcID.getLevel());
						npc.setMaxHp(npcID.getHp());
						npc.setCurrentHp(npcID.getHp());
						npc.setPhysicalAttack(npcID.getPatk());
						npc.setPhysicalDefense(npcID.getPdef());
						npc.setMagicalAttack(npcID.getMatk());
						npc.setMagicalDefense(npcID.getMdef());
						npc.setExpReward(npcID.getExp());
						npc.setSpReward(npcID.getSp());
						npc.setAttackRange(npcID.getAttackrange());
						npc.setAggressive(npcID.getAgro() == 1);
					}
					else
					{
						_log.warning("npc data is missing in npc2.csv for npc id "+tempId+" using defaults");
						npc.setLevel(10);
						npc.setMaxHp(200);
						npc.setCurrentHp(200);
						npc.setPhysicalAttack(50);
						npc.setPhysicalDefense(100);
						npc.setMagicalAttack(50);
						npc.setMagicalDefense(100);
						npc.setExpReward(100);
						npc.setSpReward(10);
						npc.setAttackRange(36); // default melee range
						npc.setAggressive(false);
					}
				}
			}
			catch (Exception e)
			{
				_log.warning("NPC class not found");
			}
			
			return npc;
		}
			
		return null;
	}

	public void storeObject(L2Object temp)
	{
		_allObjects.put(new Integer(temp.getObjectId()), temp);
	}
	
	public void removeObject(L2Object object)
	{
		_allObjects.remove(new Integer(object.getObjectId()));  // suggestion by whatev
	}

	
	/**
	 * find the object that belongs to an ID
	 * @param oID
	 * @return  null if no object was found. 
	 */
	public L2Object findObject(int oID)
	{
		return (L2Object) _allObjects.get(new Integer(oID));
	}
	
	public void addVisibleObject(L2Object object)
	{
		L2Object[] visible = getVisibleObjects(object, 2000);
		_log.finest("objects in range:"+visible.length);
		
		// tell the player about the surroundings
		for (int i = 0; i < visible.length; i++)
		{
			object.addKnownObject(visible[i]);
			if (object instanceof L2ItemInstance && visible[i].getKnownObjects().contains(object))
			{
				// special case for droped items. they are already known when they appear in the world
			}
			else
			{
				visible[i].addKnownObject(object);
			}
		}
		
		
		_visibleObjects.put(new Integer(object.getObjectId()), object);
		if (object instanceof L2PcInstance)
		{
			_allPlayers.put(((L2PcInstance)object).getName(), object);
		}
	}

	public void removeVisibleObject(L2Object object)
	{
		_visibleObjects.remove(new Integer(object.getObjectId()));
		_log.fine("World has now " + _visibleObjects.size() + " visible objects");

		Object[] temp = object.getKnownObjects().toArray();
		for (int i = 0; i < temp.length; i++)
		{
			L2Object temp1 = (L2Object) temp[i];
			temp1.removeKnownObject(object);
			object.removeKnownObject(temp1);
		}

		if (object instanceof L2PcInstance)
		{
			_allPlayers.remove(((L2PcInstance)object).getName());
		}
	}
	
	/**
	 * returns all visible objects in range
	 * @param object
	 * @param radius
	 * @return
	 */
	public L2Object[] getVisibleObjects(L2Object object, int radius)
	{
		int x = object.getX();
		int y = object.getY();
		int sqRadius = radius*radius;
		ArrayList result = new ArrayList();
		for (Iterator iter = _visibleObjects.values().iterator(); iter.hasNext();)
		{
			L2Object element = (L2Object) iter.next();
			
			if (element.equals(object)) continue;	// skip our own character
			
			int x1 = element.getX();
			int y1 = element.getY();
			
			long dx = x1 - x;
			long dy = y1 - y;
			long sqDist = dx*dx + dy*dy;
			if (sqDist < sqRadius)
			{
				result.add(element);
			}
		}
		
		return (L2Object[]) result.toArray(new L2Object[result.size()]);
	}
	
	public L2PcInstance[] getAllPlayers()
	{
		return (L2PcInstance[]) _allPlayers.values().toArray(new L2PcInstance[_allPlayers.size()]);
	}
	
	public L2PcInstance getPlayer(String name)
	{
		return (L2PcInstance) _allPlayers.get(name);
	}
}
