/*
 * $Header: /var/lib/gforge/chroot/cvsroot/nextg/CVSROOT/L2_Gameserver/java/net/sf/l2j/gameserver/serverpackets/SystemMessage.java,v 1.3 2004/08/05 00:53:40 nextg Exp $
 *
 * $Author: nextg $
 * $Date: 2004/08/05 00:53:40 $
 * $Revision: 1.3 $
 * $Log: SystemMessage.java,v $
 * Revision 1.3  2004/08/05 00:53:40  nextg
 * RuSh
 *
 * Revision 1.5  2004/07/17 11:26:28  l2chef
 * some refactoring
 *
 * Revision 1.4  2004/07/04 17:38:25  l2chef
 * level up message added
 *
 * Revision 1.3  2004/07/04 11:14:53  l2chef
 * logging is used instead of system.out
 *
 * Revision 1.2  2004/06/27 08:51:42  jeichhorn
 * Added copyright notice
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.gameserver.serverpackets;

import java.io.IOException;
import java.util.Vector;

/**
 * This class ...
 * 
 * @version $Revision: 1.3 $ $Date: 2004/08/05 00:53:40 $
 */
public class SystemMessage extends ServerBasePacket
{
	// d d (d S/d d/d dd)
	//      |--------------> 0 - String  1-number 2-textref npcname (1000000-1002655)  3-textref itemname 4-textref skills 5-??	
	private static final int TYPE_SKILL_NAME = 4;
	private static final int TYPE_ITEM_NAME = 3;
	private static final int TYPE_NPC_NAME = 2;
	private static final int TYPE_NUMBER = 1;
	private static final int TYPE_TEXT = 0;
	private static final String _S__7A_SYSTEMMESSAGE = "[S] 7A SystemMessage";
	private int _messageId;
	private Vector _types = new Vector();
	private Vector _values = new Vector();
	
	public static int NOT_ENOUGH_MP = 24;
	
	public static int YOU_DID_S1_DMG = 0x23;
	public static int S1_GAVE_YOU_S2_DMG = 0x24;
	public static int MISSED_TARGET = 0x2b;
	public static int CRITICAL_HIT = 0x2c;
	public static int EFFECT_S1_DISAPPEARED = 0x5c;
	public static int YOU_EARNED_S1_EXP_AND_S2_SP = 0x5f;
	public static int YOU_INCREASED_YOUR_LEVEL = 0x60;
	
	public static int ITEM_MISSING_TO_LEARN_SKILL = 0x114;
	public static int LEARNED_SKILL_S1 = 0x115;
	public static int NOT_ENOUGH_SP_TO_LEARN_SKILL = 0x116;
	
	public static int FALL_DAMAGE_S1 = 0x128;
	public static int DROWN_DAMAGE_S1 = 0x129;

	public static int YOU_DROPPED_S1 = 0x12a;
	public static int YOU_PICKED_UP_S1_S2 = 0x12b;
	public static int YOU_PICKED_UP_S1 = 0x12c;

	public static int SOULSHOTS_GRADE_MISMATCH = 0x151;
	public static int NOT_ENOUGH_SOULSHOTS = 0x152;
	public static int CANNOT_USE_SOULSHOTS = 0x153;
	
	// party related 
	
	public static int S1_INVITED_YOU_TO_PARTY_FINDER_KEEPER = 572;
	public static int S1_INVITED_YOU_TO_PARTY_RANDOM = 573;
	
	public static int S1_INVITED_YOU_TO_PARTY = 66;
	public static int YOU_INVITED_S1_TO_PARTY = 105;
	
	public static int OTHER_PARTY_IS_DROZEN = 692;
	
	public static int YOU_JOINED_S1_PARTY = 106;
	public static int S1_JOINED_PARTY = 107;
	public static int S1_LEFT_PARTY = 108;
	public static int YOU_LEFT_PARTY = 200;

	public static int S1_DID_NOT_REPLY = 135;
	public static int YOU_DID_NOT_REPLY = 136;
	
	public static int PLAYER_DECLINED = 305;
	
	public static int PARTY_FULL = 155;
	public static int S1_IS_ALREADY_IN_PARTY = 160;
	public static int INVITED_USER_NOT_ONLINE = 161;

	public static int PARTY_DISPERSED = 203;

	//weight & inventory
	public static int WEIGHT_LIMIT_EXCEEDED = 422;
	

	
	

	

	/**
	 * @param _characters
	 */
	public SystemMessage(int messageId)
	{
		_messageId = messageId;
	}
	
	public void addString(String text)
	{
		_types.add(new Integer(TYPE_TEXT));
		_values.add(text);
	}

	public void addNumber(int number)
	{
		_types.add(new Integer(TYPE_NUMBER));
		_values.add(new Integer(number));
	}
	
	public void addNpcName(int id)
	{
		_types.add(new Integer(TYPE_NPC_NAME));
		_values.add(new Integer(1000000+id));
	}

	public void addItemName(int id)
	{
		_types.add(new Integer(TYPE_ITEM_NAME));
		_values.add(new Integer(id));
	}

	public void addSkillName(int id)
	{
		_types.add(new Integer(TYPE_SKILL_NAME));
		_values.add(new Integer(id));
	}

	public byte[] getContent() throws IOException
	{
		writeC(0x7a);

		writeD(_messageId);
		writeD(_types.size());

		for (int i=0; i<_types.size();i++)
		{
			int t = ((Integer)_types.get(i)).intValue();


			writeD(t);

			switch (t)
			{
				case TYPE_TEXT:
				{
					writeS( (String)_values.get(i));
					break;
				}
				case TYPE_NUMBER:
				case TYPE_NPC_NAME:
				case TYPE_ITEM_NAME:
				{
					int t1 = ((Integer)_values.get(i)).intValue();
					writeD(t1);	
					
					break;
				}
				case TYPE_SKILL_NAME:
				{
					int t1 = ((Integer)_values.get(i)).intValue();
					writeD(t1);	
					writeD(1);  //??
				}
			}
		}

		return getBytes();
	}

	/* (non-Javadoc)
	 * @see net.sf.l2j.gameserver.serverpackets.ServerBasePacket#getType()
	 */
	public String getType()
	{
		return _S__7A_SYSTEMMESSAGE;
	}
}
	
