/*
 * $Header: /var/lib/gforge/chroot/cvsroot/nextg/CVSROOT/L2_Gameserver/java/net/sf/l2j/loginserver/LoginServer.java,v 1.3 2004/08/05 00:53:40 nextg Exp $
 *
 * $Author: nextg $
 * $Date: 2004/08/05 00:53:40 $
 * $Revision: 1.3 $
 * $Log: LoginServer.java,v $
 * Revision 1.3  2004/08/05 00:53:40  nextg
 * RuSh
 *
 * Revision 1.7  2004/07/19 22:22:32  j3ster
 * if external hostname is not set (default), use server's local ip (not 127.0.0.1)
 *
 * Revision 1.6  2004/07/09 20:17:51  l2chef
 * workaround to allow router usage and local use at the same time. contributed by  whatev
 *
 * Revision 1.5  2004/06/30 21:51:34  l2chef
 * using jdk logger instead of println
 *
 * Revision 1.4  2004/06/29 22:55:00  l2chef
 * binding ips can be defined independent of public server hostname
 *
 * Revision 1.3  2004/06/27 20:57:14  l2chef
 * autoAccount creation can be set in config file
 *
 * Revision 1.2  2004/06/27 08:51:43  jeichhorn
 * Added copyright notice
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.loginserver;

import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Properties;
import java.util.logging.Logger;

/**
 * This class ...
 * 
 * @version $Revision: 1.3 $ $Date: 2004/08/05 00:53:40 $
 */
public class LoginServer extends Thread
{
	private LoginController _loginController;
	private ServerSocket _serverSocket;
	private Logins _logins;
	private String _ip;
	private int _port;
	private int _gamePort;
	private String _externalHostname; 
	static Logger _log = Logger.getLogger(LoginServer.class.getName());

	public static void main(String[] args) throws IOException
	{
		LoginServer server = new LoginServer();
		_log.config("LoginServer Listening on port 2106");
		server.start();
	}
	
	/**
	 * 
	 */
	public void run()
	{
		while (true)
		{
			try
			{
				_log.fine("waiting for client connection...");
				Socket connection = _serverSocket.accept();
				_log.fine("connection from "+connection.getInetAddress());
				
				// use local ip addess if no external hostname is set or if
				// the the connection is requested from localhost (for router compatibility)
				if (_externalHostname == null || "localhost".equals( connection.getInetAddress().getHostName()))
				{
					_log.fine("using local ip as server ip "+InetAddress.getLocalHost().getHostAddress());
					new ClientThread(connection, _logins, InetAddress.getLocalHost().getHostAddress(), _gamePort);
				}
				else
				{
					_log.fine("using external ip as server ip");
					new ClientThread(connection, _logins, _externalHostname, _gamePort);
				}
			} 
			catch (IOException e)
			{
				// not a real problem
			}
		}
	}

	public LoginServer() throws IOException
	{
		super("LoginServer");

		Properties serverSettings = new Properties();
		InputStream is = getClass().getResourceAsStream("/server.cfg");  
		serverSettings.load(is);
		is.close();

		String loginIp = serverSettings.getProperty("LoginserverHostname");
		_externalHostname = serverSettings.getProperty("ExternalHostname");
		String gamePort = serverSettings.getProperty("GameserverPort");
		String createAccounts = serverSettings.getProperty("AutoCreateAccounts");
		
		
		if (!"*".equals(loginIp))
		{
			InetAddress adr = InetAddress.getByName(loginIp);
			_ip = adr.getHostAddress();
			_log.config("LoginServer listening on IP:"+_ip + " Port 2106");
			_serverSocket = new ServerSocket(2106, 50, adr);
		}
		else
		{
			_log.config("LoginServer listening on all available IPs on Port 2106");
			_serverSocket = new ServerSocket(2106);
		}

		_log.config("External Hostname is: "+_externalHostname);
		
		_logins = new Logins( Boolean.valueOf(createAccounts).booleanValue() );
		
		_gamePort = Integer.parseInt(gamePort);
	}
}
