/*
 * $Header: /var/lib/gforge/chroot/cvsroot/nextg/CVSROOT/L2_Tools/java/net/sf/l2j/crypt/CryptInputStream.java,v 1.1 2004/08/04 10:39:36 nextg Exp $
 *
 * $Author: nextg $
 * $Date: 2004/08/04 10:39:36 $
 * $Revision: 1.1 $
 * $Log: CryptInputStream.java,v $
 * Revision 1.1  2004/08/04 10:39:36  nextg
 * a
 *
 * Revision 1.2  2004/06/28 22:09:41  l2chef
 * crypt streams behave now exactly as the client. end of files are not crypted any more
 *
 * Revision 1.1  2004/06/27 20:02:26  l2chef
 * init
 *
 * Revision 1.2  2004/06/27 08:12:59  jeichhorn
 * Added copyright notice
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.crypt;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * This class ...
 * 
 * @version $Revision: 1.1 $ $Date: 2004/08/04 10:39:36 $
 */
public class CryptInputStream extends InputStream
{
	private static final String UTF_LE = "UTF-16LE";
	
	InputStream _in;
	int _counter;
	int _maxLength;
	
	boolean _crypted;
	boolean _headerMatch;
	int _version;
	int[] _header = {
			0x4c,0x00,0x69,0x00,0x6e,0x00,0x65,0x00,
			0x61,0x00,0x67,0x00,0x65,0x00,0x32,0x00,
			0x56,0x00,0x65,0x00,0x72,0x00
	};		// Lineage2Ver
	
	byte[] _inBuffer;
	byte[] _outBuffer;
	int _bytesAvailable;
	int _offset;
	BlowfishEngine _blow;
	int _xor = -1;
	private ByteArrayOutputStream _baos;
	
	/**
	 * @param in
	 */
	public CryptInputStream(InputStream in)
	{
		_in = in;
		try
		{
			// this will not work with all inputstreams
			_maxLength = in.available();
		} 
		catch (IOException e)
		{
			// we can ignore this. it will work anyway
		}
		
		_headerMatch = true;
		_counter = 0;
		_baos = new ByteArrayOutputStream();
	}
	
	
	/* (non-Javadoc)
	 * @see java.io.InputStream#read()
	 */
	public int read() throws IOException
	{
		if (_counter == 0)
		{
			_inBuffer = new byte[28];
			_in.read(_inBuffer);
		}
		 
		while (_headerMatch == true && _counter <22)
		{
			_headerMatch = (_inBuffer[_counter] == _header[_counter]);
			_counter++;
		}
		
		while (_counter >21 && _counter <28 && _headerMatch)
		{
			if (_inBuffer[_counter] > 0)
			{
				_version = _version*10 + (_inBuffer[_counter] - 0x30);
				_crypted = true;
			}
			_counter++;
		}
		
		if (_counter == 28 &&_crypted)
		{
			_inBuffer = new byte[8];
			_outBuffer = new byte[8];
			String key = "";
			
			if (_version == 211)
			{
				key = "31==-%&@!^+][;'.]94-\000";
				initBlowFish(key);
			}
			else if (_version == 212)
			{	
				key = "[;'.]94-&@%!^+]-31==\000";
				initBlowFish(key);
			}
			else if (_version == 111)
			{
				_xor = 0xac;
			}
			
			_counter++;
		}

		int value = -1;
		// we have reached the version/crc block .. work completed
		if (_counter == _maxLength-19)
		{
			return value;
		}

		if (_crypted && _blow != null)
		{
			if (_bytesAvailable-_offset == 0)
			{
				_offset = 0;
				if ((_bytesAvailable = _in.read(_inBuffer)) != -1)
				{
					if (_counter < _maxLength-28)
					{
						_blow.processBlock(_inBuffer,0,_outBuffer,0);
					}
					else
					{
						// last bytes are not encrypted if less than 8byte,
						// seems to be a bug from ncsoft
						_outBuffer = _inBuffer;
					}
				}
			}

			if (_bytesAvailable > 0)
			{
				value = _outBuffer[_offset++] & 0xff;
			}
			else
			{
				// end of stream
				value = -1;
			}
			
		}
		else
		{
			value = _in.read();
			if (value != -1 && _xor != -1)
			{
		        value = (value^_xor) & 0xff;
			}
		}

		_counter++;
		return value;
	}


	public String readLine() throws IOException
	{
		_baos.reset();
		boolean lineEnd = false;
		boolean fileEnd = false;
		int hi = 0;
		int lo = 0;
		
		while( !lineEnd && !fileEnd)
		{
			hi = read();
			lo = read();
			
			lineEnd = (hi == 0x0a && lo == 0x00);
			fileEnd = (hi == -1 || lo == -1); 
			
			if (!lineEnd && ! fileEnd)
			{
				_baos.write(hi);
				_baos.write(lo);
			}
		}
		
		if (_baos.size() == 0 || fileEnd)
		{
			return null;
		}
		else
		{
			return new String(_baos.toByteArray(),UTF_LE);
		}
	}
	
	
	private void initBlowFish(String key)
	{
		_blow = new BlowfishEngine();
		
		byte[] keybytes = key.getBytes();
		_blow.init(false, keybytes);
	}
}
