/*
 * $Header: /var/lib/gforge/chroot/cvsroot/nextg/CVSROOT/L2_Tools/java/net/sf/l2j/crypt/CryptOutputStream.java,v 1.1 2004/08/04 10:39:36 nextg Exp $
 *
 * $Author: nextg $
 * $Date: 2004/08/04 10:39:36 $
 * $Revision: 1.1 $
 * $Log: CryptOutputStream.java,v $
 * Revision 1.1  2004/08/04 10:39:36  nextg
 * a
 *
 * Revision 1.2  2004/06/28 22:09:41  l2chef
 * crypt streams behave now exactly as the client. end of files are not crypted any more
 *
 * Revision 1.1  2004/06/27 20:02:26  l2chef
 * init
 *
 * Revision 1.2  2004/06/27 08:12:59  jeichhorn
 * Added copyright notice
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.crypt;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

/**
 * This class ...
 * 
 * @version $Revision: 1.1 $ $Date: 2004/08/04 10:39:36 $
 */

public class CryptOutputStream extends OutputStream
{
	private static final String UTF_LE = "UTF-16LE";
	
	OutputStream _out;
	
	int _counter;
	boolean _crypted;
	boolean _headerMatch;
	int _version;
	int[] _header = {
			0x4c,0x00,0x69,0x00,0x6e,0x00,0x65,0x00,
			0x61,0x00,0x67,0x00,0x65,0x00,0x32,0x00,
			0x56,0x00,0x65,0x00,0x72,0x00
	};		// Lineage2Ver
	
	byte[] _inBuffer;
	int _inBufferCount;
	
	byte[] _outBuffer;
	int _bytesAvailable;
	int _offset;
	BlowfishEngine _blow;
	int _xor = -1;
	private ByteArrayOutputStream _baos;


	/**
	 * @param out
	 * @throws IOException
	 * @throws UnsupportedEncodingException
	 */
	public CryptOutputStream(OutputStream out, int cryptVersion) throws IOException
	{
		_out = out;
		_version = cryptVersion;

		String key = "";
		if (_version == 212)
		{	
			out.write("Lineage2Ver212".getBytes("UTF-16LE"));
			key = "[;'.]94-&@%!^+]-31==\000";
		}
		else if (_version == 211)
		{
			out.write("Lineage2Ver211".getBytes("UTF-16LE"));
			key = "31==-%&@!^+][;'.]94-\000";
		}
		else if (_version == 111)
		{
			out.write("Lineage2Ver111".getBytes("UTF-16LE"));
			_xor = 0xac;
		}
		else
		{
			throw new IOException("Unsupported cryptVersion");
		}
			

		if (_version > 200)
		{
			_inBuffer = new byte[8];
			_outBuffer = new byte[8];
			_blow = new BlowfishEngine();
			byte[] keybytes = key.getBytes();
			_blow.init(true, keybytes);
		}
	}


	/* (non-Javadoc)
	 * @see java.io.OutputStream#write(int)
	 */
	public void write(int b) throws IOException
	{
		if (_version == 111)
		{
			int value = b ^ _xor;
			_out.write(value);
		}
		else if (_version > 200)
		{
			_inBuffer[_inBufferCount++] =(byte) b;
			if (_inBufferCount == 8)
			{
				_blow.processBlock(_inBuffer, 0, _outBuffer, 0);
				_out.write(_outBuffer, 0, 8);
				_inBufferCount = 0;
			}
		}
	}
	
	
	
	public void close() throws IOException
	{
		if (_version >200 && _inBufferCount > 0)
		{
			// write out any remaining bytes  
			// (this would be correct but ncsoft has a bug, so that the remaining bytes dont get encrypted)
//			_blow.processBlock(_inBuffer, 0, _outBuffer, 0);
//			_out.write(_outBuffer, 0, _inBufferCount);
			
			
			_out.write(_inBuffer, 0, _inBufferCount);
			_inBufferCount = 0;
		}
		
		// this block contains version and CRC infos
		// the client doesnt currently check the CRC, so we just leave it empty
		// might be required in future versions
		byte[] empty = new byte[20];
		_out.write(empty);
		_out.close();
		super.close();
	}
	
	private void initBlowFish(String key)
	{
		_blow = new BlowfishEngine();
		
		byte[] keybytes = key.getBytes();
		_blow.init(false, keybytes);
	}
}
