/*
 * $Header: /var/lib/gforge/chroot/cvsroot/nextg/CVSROOT/L2_Tools/java/net/sf/l2j/dataextraction/DataFileConversion.java,v 1.1 2004/08/04 10:39:35 nextg Exp $
 *
 * $Author: nextg $
 * $Date: 2004/08/04 10:39:35 $
 * $Revision: 1.1 $
 * $Log: DataFileConversion.java,v $
 * Revision 1.1  2004/08/04 10:39:35  nextg
 * a
 *
 * Revision 1.3  2004/07/25 11:01:37  l2chef
 * classnames from lineage client are converted to L2J handler classes. might not be correct in all cases
 *
 * Revision 1.2  2004/06/27 21:00:20  l2chef
 * fixed weapon.csv filename
 *
 * Revision 1.1  2004/06/27 20:02:27  l2chef
 * init
 *
 * Revision 1.2  2004/06/27 08:12:59  jeichhorn
 * Added copyright notice
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.dataextraction;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.HashMap;
import java.util.StringTokenizer;

import net.sf.l2j.crypt.CryptInputStream;

/**
 * This class ...
 * 
 * @version $Revision: 1.1 $ $Date: 2004/08/04 10:39:35 $
 */
public class DataFileConversion
{
	private HashMap _materials;
	private HashMap _crystalTypes;
	private HashMap _slots;
	private HashMap _skillNames;
	private HashMap _npcType;

	public static void main(String[] args) throws IOException
	{
		File installFolder = null;
		
		if (args.length < 1)
		{
			System.out.println("L2 install folder was not given as parameter.");
			System.out.print("Enter the full path: ");
			LineNumberReader isr = new LineNumberReader(new InputStreamReader(System.in));
			String path = isr.readLine();
			installFolder = new File(path);
		}
		else
		{
			installFolder = new File(args[0]);
		}

		if (!installFolder.exists())
		{
			System.out.println("Folder " + installFolder.getAbsolutePath() + " does not exist");
			return;
		}
		
		if (!installFolder.isDirectory())
		{
			System.out.println("" + installFolder.getAbsolutePath() + " is not a directory");
			return;
		}
		
		File check = new File(installFolder, "LineageII.exe");
		if (!check.exists())
		{
			System.out.println("LineageII.exe was not found. is the folder correct ?");
			return;
		}
		
		new DataFileConversion(installFolder);
	}
	
	public DataFileConversion(File l2folder) throws IOException
	{
		File dataFolder = new File(l2folder, "system");
		File outputFolder = new File("data");
		outputFolder.mkdir();
		convertWeaponGrp(dataFolder);
		convertArmorGrp(dataFolder);
		convertEtcItemGrp(dataFolder);
		convertSkillGrp(dataFolder);
		convertNpcGrp(dataFolder);
	}

	private void convertNpcGrp(File dataFolder) throws FileNotFoundException, IOException
	{
		FileInputStream fis = new FileInputStream(new File(dataFolder, "npcgrp.txt"));
		BufferedInputStream bin = new BufferedInputStream(fis);
		CryptInputStream din = new CryptInputStream(bin);

		FileWriter fos = new FileWriter("data/npc.csv");
		fos.write("#id;name;class;collision_radius;collision_height\r\n");
		String line = null;
		int count=0;
		
		while ( (line = din.readLine()) != null)
		{
			String result = parseNpcLine(line);
			fos.write(result+"\r\n");
			count++;
		}
		
		din.close();
		fos.close();
		System.out.println("converted " + count + " NPCs");
	}


	private void convertWeaponGrp(File dataFolder) throws FileNotFoundException, IOException
	{
		FileInputStream fis = new FileInputStream(new File(dataFolder, "weapongrp.txt"));
		BufferedInputStream bin = new BufferedInputStream(fis);
		CryptInputStream din = new CryptInputStream(bin);
		
		FileWriter fos = new FileWriter("data/weapon.csv");
		fos.write("#id;name;bodypart;crystallizable;weight;soulshots;spiritshots;material;crystal;p.dam;rnd.dam;weaponType;critical;hit.modify;avoid.modify;shield.def;shield.def.rate;atk.speed;mp.consume;m.dam;durability\r\n");
		String line = null;
		int count=0;
		
		while ( (line = din.readLine()) != null)
		{
			String result = parseWeaponLine(line);
			fos.write(result+"\r\n");
			count++;
		}
		
		din.close();
		fos.close();
		System.out.println("converted " + count + " Weapons");
	}

	private void convertArmorGrp(File dataFolder) throws FileNotFoundException, IOException
	{
		FileInputStream fis = new FileInputStream(new File(dataFolder, "armorgrp.txt"));
		BufferedInputStream bin = new BufferedInputStream(fis);
		CryptInputStream din = new CryptInputStream(bin);
		
		FileWriter fos = new FileWriter("data/armor.csv");
		fos.write("#id;name;bodypart;crystallizable;armor.type;weight;material;crystal.type;avoid.modify;durability;p.def;m.def;mp.bonus\r\n");
		String line = null;
		int count=0;
		
		while ( (line = din.readLine()) != null)
		{
			String result = parseArmorLine(line);
			fos.write(result+"\r\n");
			count++;
		}
		
		din.close();
		fos.close();
		System.out.println("converted " + count + " Armor");
	}

	private void convertEtcItemGrp(File dataFolder) throws FileNotFoundException, IOException
	{
		FileInputStream fis = new FileInputStream(new File(dataFolder, "etcitemgrp.txt"));
		BufferedInputStream bin = new BufferedInputStream(fis);
		CryptInputStream din = new CryptInputStream(bin);
		
		FileWriter fos = new FileWriter("data/etcitem.csv");
		fos.write("#id;name;crystallizable;item.type;weight;consume.type;material;crystal.type;durability\r\n");
		String line = null;
		int count=0;
		
		while ( (line = din.readLine()) != null)
		{
			String result = parseEtcItemLine(line);
			fos.write(result+"\r\n");
			count++;
		}
		
		din.close();
		fos.close();
		System.out.println("converted " + count + " EtcItems");
	}

	private void convertSkillGrp(File dataFolder) throws FileNotFoundException, IOException
	{
		_skillNames = new HashMap();
		FileInputStream fis1 = new FileInputStream(new File(dataFolder, "skillname-e.txt"));
		BufferedInputStream bin1 = new BufferedInputStream(fis1);
		CryptInputStream din1 = new CryptInputStream(bin1);
		String line = null;
		
		while ( (line = din1.readLine()) != null)
		{
			parseSkillNameLine(line);
		}
		
		din1.close();
		
		
		FileInputStream fis = new FileInputStream(new File(dataFolder, "skillgrp.txt"));
		BufferedInputStream bin = new BufferedInputStream(fis);
		CryptInputStream din = new CryptInputStream(bin);
		
		FileWriter fos = new FileWriter("data/skills.csv");
		fos.write("#id;name;level;operateType;isMagic;mpConsume;hpConsume;itemComsumeId;itemConsumeCount;castRange;skillTime;reuseDelay;buffDuration;hitTime\r\n");
		int count=0;
		
		while ( (line = din.readLine()) != null)
		{
			String result = parseSkillLine(line);
			fos.write(result+"\r\n");
			count++;
		}
		
		din.close();
		fos.close();
		System.out.println("converted " + count + " skills");
	}
	
	//
	//  skills
	//
	private String parseSkillLine(String line)
	{
		StringTokenizer	st = new StringTokenizer(line, "\t\r\n");
		StringBuffer result = new StringBuffer();
		
		while (st.hasMoreTokens())
		{
			String col = parseSkillColumns(st.nextToken());
			if (col != null)
			{
				result.append(col+";");
			}
		}
		
		return result.toString();
	}

	private void parseSkillNameLine(String line)
	{
		StringTokenizer	st = new StringTokenizer(line, "\t\r\n");
		String key = null;
		String name = null;
		
		while (st.hasMoreTokens())
		{
			String column = st.nextToken();
			StringTokenizer st2 = new StringTokenizer(column, "=");
			while (st2.hasMoreTokens())
			{
				String id = st2.nextToken().trim();
				
				if (id.equals("skill_id"))
				{
					key = st2.nextToken().trim();
				}
//			else if (id.equals("skill_level"))
//			{
//				key += "," + st.nextToken();
//			}
				else if (id.equals("name"))
				{
					name = st2.nextToken().replace('[',' ').replace(']',' ').trim();
				}
			}
		}
		_skillNames.put(key, name);
	}
	
	private void parseSkillNameColumns(String column)
	{
		String key = null;
		String name = null;
	
		StringTokenizer st = new StringTokenizer(column, "= ");
		while (st.hasMoreTokens())
		{
			String id = st.nextToken().trim();
			
			if (id.equals("skill_id"))
			{
				key = st.nextToken();
			}
//			else if (id.equals("skill_level"))
//			{
//				key += "," + st.nextToken();
//			}
			else if (id.equals("name"))
			{
				name = st.nextToken().replace('[',' ').replace(']',' ').trim();
			}
		}
		_skillNames.put(key, name);
	}
	
	private String parseSkillColumns(String column)
	{
		String result = null;
		
		StringTokenizer st = new StringTokenizer(column, "= ");
		while (st.hasMoreTokens())
		{
			String id = st.nextToken().trim();
			
			if (id.equals("skill_id"))
			{
				result = st.nextToken();
				String name = (String) _skillNames.get(result);
				result += ";"+name;
			}
			else if (id.equals("skill_level"))
			{
				result = st.nextToken();
			}
			else if (id.equals("operate_type"))
			{
				int type=Integer.parseInt(st.nextToken());
				switch (type)
				{
					case 0:
						result = "once";
						break;
					case 1:
						result = "duration";
						break;
					case 2:
						result = "always";//passive
						break;
					case 3:
						result = "toggle";
						break;
					default:
						result = "unknown";
				}
			}
			else if (id.equals("is_magic"))
			{
				result = "" + st.nextToken().trim().equals("1");
			}
			else if (id.equals("mp_consume"))
			{
				result = "" + (int) Float.parseFloat(st.nextToken());
			}
			else if (id.equals("hp_consume"))
			{
				result = "" + Integer.parseInt(st.nextToken());
			}
			else if (id.equals("consume_item_id"))
			{
				result = st.nextToken();
			}
			else if (id.equals("consume_item_count"))
			{
				result = st.nextToken();
			}
			else if (id.equals("cast_range"))
			{
				result = st.nextToken();
			}
			else if (id.equals("skill_time"))
			{
				result = "" + (int) Float.parseFloat(st.nextToken())*1000;
			}
			else if (id.equals("reuse_delay"))
			{
				result = "" + (int) Float.parseFloat(st.nextToken())*1000;
			}
			else if (id.equals("abnormal_time"))
			{
				result = "" + (int) Float.parseFloat(st.nextToken())*1000;
			}
			else if (id.equals("hit_time"))
			{
				result = "" + (int) Float.parseFloat(st.nextToken())*1000;
			}
		}
		
		return result;
	}
	
	
	
	//
	//  NPCs
	//
	private String parseNpcLine(String line)
	{
		StringTokenizer	st = new StringTokenizer(line, "\t\r\n");
		StringBuffer result = new StringBuffer();
		
		while (st.hasMoreTokens())
		{
			String col = parseNpcColumns(st.nextToken() );
			if (col != null)
			{
				result.append(col+";");
			}
		}
		
		return result.toString();
	}
	
	private String parseNpcColumns(String column)
	{
		String result = null;
		
		StringTokenizer st = new StringTokenizer(column, "= ");
		while (st.hasMoreTokens())
		{
			String id = st.nextToken().trim();
			
			if (id.equals("npc_id"))
			{
				result = "" + (Integer.parseInt(st.nextToken()) + 1000000);
			}
			else if (id.equals("npc_name"))
			{
				result = st.nextToken().replace('[',' ').replace(']',' ').trim();
			}
			else if (id.equals("class_name"))
			{
				String name = st.nextToken().replace('[',' ').replace(']',' ').trim().toLowerCase();
				if (name.startsWith("lineagemonster"))
				{
					result = "L2Monster";
				}
				else if (name.startsWith("lineagenpc"))
				{
					if (name.indexOf("guard") != -1)
					{
						result = "L2Guard";
					}
					else if (name.indexOf("teacher") != -1)
					{
						result = "L2Trainer";
					}
					else if (name.indexOf("trader") != -1)
					{
						result = "L2Merchant";
					}
					else if (name.indexOf("teleporter") != -1)
					{
						result = "L2Teleporter";
					}
					else
					{
						result = "L2Npc";
					}
				}
				else
				{
					result = "L2Npc";
				}
			}
			else if (id.equals("collision_radius"))
			{
				double radius = Double.parseDouble(st.nextToken());
				result = "" + radius;
			}
			else if (id.equals("collision_height"))
			{
				double height = Double.parseDouble(st.nextToken());
				result = "" + height;
			}
		}
		
		return result;
	}

	
	
	//
	//  Weapons
	//
	private String parseWeaponLine(String line)
	{
		StringTokenizer	st = new StringTokenizer(line, "\t\r\n");
		StringBuffer result = new StringBuffer();
		
		while (st.hasMoreTokens())
		{
			String col = parseWeaponColumns(st.nextToken() );
			if (col != null)
			{
				result.append(col+";");
			}
		}
		
		return result.toString();
	}
	
	private String parseWeaponColumns(String column)
	{
		String result = null;
		
		StringTokenizer st = new StringTokenizer(column, "= ");
		while (st.hasMoreTokens())
		{
			String id = st.nextToken().trim();
			
			if (id.equals("object_id"))
			{
				result = st.nextToken();
			}
			else if (id.equals("object_name"))
			{
				result = st.nextToken().replace('[',' ').replace(']',' ').trim();
			}
			else if (id.equals("body_part"))
			{
				result = st.nextToken().replace(';',',').replace('{',' ').replace('}',' ').trim();
			}
			else if (id.equals("crystallizable"))
			{
				if(st.nextToken().equals("1"))
				{
					result = "true";
				}
				else
				{
					result = "false";
				}
			}
			else if (id.equals("weight"))
			{
				int weight = Integer.parseInt(st.nextToken());
				result = "" + weight;
			}
			else if (id.equals("soulshot_count"))
			{
				int ss = Integer.parseInt(st.nextToken());
				result = "" + ss;
			}
			else if (id.equals("spiritshot_count"))
			{
				int ss = Integer.parseInt(st.nextToken());
				result = "" + ss;
			}
			else if (id.equals("material_type"))
			{
				result=st.nextToken();
			}
			else if (id.equals("crystal_type"))
			{
				result=st.nextToken();
			}
			else if (id.equals("physical_damage"))
			{
				result=st.nextToken();
			}
			else if (id.equals("random_damage"))
			{
				result=st.nextToken();
			}
			else if (id.equals("weapon_type"))
			{
				result=st.nextToken();
			}
			else if (id.equals("critical"))
			{
				result=st.nextToken();
			}
			else if (id.equals("hit_modify"))
			{
				result=st.nextToken();
			}
			else if (id.equals("avoid_modify"))
			{
				result=st.nextToken();
			}
			else if (id.equals("shield_defense"))
			{
				result=st.nextToken();
			}
			else if (id.equals("shield_defense_rate"))
			{
				result=st.nextToken();
			}
			else if (id.equals("attack_speed"))
			{
				result=st.nextToken();
			}
			else if (id.equals("mp_consume"))
			{
				result=st.nextToken();
			}
			else if (id.equals("magical_damage"))
			{
				result=st.nextToken();
			}
			else if (id.equals("durability"))
			{
				result=st.nextToken();
			}
			
		}
		
		return result;
	}	

	
	
	//
	//  Armor
	//
	private String parseArmorLine(String line)
	{
		StringTokenizer	st = new StringTokenizer(line, "\t\r\n");
		StringBuffer result = new StringBuffer();
		
		while (st.hasMoreTokens())
		{
			String col = parseArmorColumns(st.nextToken() );
			if (col != null)
			{
				result.append(col+";");
			}
		}
		
		return result.toString();
	}
	
	private String parseArmorColumns(String column)
	{
		String result = null;
		
		StringTokenizer st = new StringTokenizer(column, "= ");
		while (st.hasMoreTokens())
		{
			String id = st.nextToken().trim();
			
			if (id.equals("object_id"))
			{
				result = st.nextToken();
			}
			else if (id.equals("object_name"))
			{
				result = st.nextToken().replace('[',' ').replace(']',' ').trim();
			}
			else if (id.equals("body_part"))
			{
				result = st.nextToken().replace(';',',').replace('{',' ').replace('}',' ').trim();
			}
			else if (id.equals("crystallizable"))
			{
				if(st.nextToken().equals("1"))
				{
					result = "true";
				}
				else
				{
					result = "false";
				}
			}
			else if (id.equals("armor_type"))
			{
				result=st.nextToken();
			}
			else if (id.equals("weight"))
			{
				int weight = Integer.parseInt(st.nextToken());
				result = "" + weight;
			}
			else if (id.equals("material_type"))
			{
				result=st.nextToken();
			}
			else if (id.equals("crystal_type"))
			{
				result=st.nextToken();
			}
			else if (id.equals("avoid_modify"))
			{
				result=st.nextToken();
			}
			else if (id.equals("durability"))
			{
				result=st.nextToken();
			}
			else if (id.equals("physical_defense"))
			{
				result=st.nextToken();
			}
			else if (id.equals("magical_defense"))
			{
				result=st.nextToken();
			}
			else if (id.equals("mp_bonus"))
			{
				result=st.nextToken();
			}
		}
		
		return result;
	}	

	
	//
	//  EtcItems
	//
	private String parseEtcItemLine(String line)
	{
		StringTokenizer	st = new StringTokenizer(line, "\t\r\n");
		StringBuffer result = new StringBuffer();
		
		while (st.hasMoreTokens())
		{
			String col = parseEtcItemColumns(st.nextToken() );
			if (col != null)
			{
				result.append(col+";");
			}
		}
		
		return result.toString();
	}
	
	private String parseEtcItemColumns(String column)
	{
		String result = null;
		
		StringTokenizer st = new StringTokenizer(column, "= ");
		while (st.hasMoreTokens())
		{
			String id = st.nextToken().trim();
			
			if (id.equals("object_id"))
			{
				result = st.nextToken();
			}
			else if (id.equals("object_name"))
			{
				result = st.nextToken().replace('[',' ').replace(']',' ').trim();
			}
			else if (id.equals("crystallizable"))
			{
				if(st.nextToken().equals("1"))
				{
					result = "true";
				}
				else
				{
					result = "false";
				}
			}
			else if (id.equals("etcitem_type"))
			{
				result=st.nextToken();
			}
			else if (id.equals("weight"))
			{
				int weight = Integer.parseInt(st.nextToken());
				result = "" + weight;
			}
			else if (id.equals("consume_type"))
			{
				result=st.nextToken().substring(13);
			}
			else if (id.equals("material_type"))
			{
				result=st.nextToken();
			}
			else if (id.equals("crystal_type"))
			{
				result=st.nextToken();
			}
			else if (id.equals("durability"))
			{
				result=st.nextToken();
			}
		}
		
		return result;
	}	

}
