/*
 * $Header: /var/lib/gforge/chroot/cvsroot/nextg/CVSROOT/L2_Tools/java/net/sf/l2j/iniupdate/IniUpdate.java,v 1.1 2004/08/04 10:39:36 nextg Exp $
 *
 * $Author: nextg $
 * $Date: 2004/08/04 10:39:36 $
 * $Revision: 1.1 $
 * $Log: IniUpdate.java,v $
 * Revision 1.1  2004/08/04 10:39:36  nextg
 * a
 *
 * Revision 1.1  2004/06/27 20:02:27  l2chef
 * init
 *
 * Revision 1.2  2004/06/27 08:12:59  jeichhorn
 * Added copyright notice
 *
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * http://www.gnu.org/copyleft/gpl.html
 */
package net.sf.l2j.iniupdate;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.OutputStream;

import net.sf.l2j.crypt.CryptInputStream;
import net.sf.l2j.crypt.CryptOutputStream;

/**
 * This class ...
 * 
 * @version $Revision: 1.1 $ $Date: 2004/08/04 10:39:36 $
 */

public class IniUpdate
{
	
	
	public static void main(String[] args) throws IOException
	{
		File installFolder = null;
		String newServer = null;
		
		if (args.length < 2)
		{
			System.out.println("L2 install folder and new Server address were not given as parameters.");
			System.out.print("Enter the full path to L2 installation: ");
			LineNumberReader isr = new LineNumberReader(new InputStreamReader(System.in));
			String path = isr.readLine();

			System.out.print("Enter new LoginServer Address: ");
			newServer = isr.readLine();
			
			installFolder = new File(path);
		}
		else
		{
			installFolder = new File(args[0]);
			newServer = args[1];
		}

		new IniUpdate(installFolder, newServer);
	}
	
	
	public IniUpdate(File installFolder, String newServer) throws FileNotFoundException, IOException
	{
		if (!installFolder.exists())
		{
			System.out.println("Folder " + installFolder.getAbsolutePath() + " does not exist");
			return;
		}
		
		if (!installFolder.isDirectory())
		{
			System.out.println("" + installFolder.getAbsolutePath() + " is not a directory");
			return;
		}
		
		File check = new File(installFolder, "LineageII.exe");
		if (!check.exists())
		{
			System.out.println("LineageII.exe was not found. is the folder correct ?");
			return;
		}
		
		File systemFolder = new File(installFolder, "system");
		File ini = new File(systemFolder, "L2.ini");
		File temp1 = new File(systemFolder, "L2.ini_temp");
		File temp2 = new File(systemFolder, "L2.ini_temp2");
		File temp3 = new File(systemFolder, "L2.ini_old");
		createBackup(ini, temp3);
		System.out.println("created backup of old ini  ... L2.ini_old");
		
		decryptIni(ini, temp1);
		System.out.println("decrypted ini");
		updateIni(temp1, temp2, newServer);
		System.out.println("updated ini");
		encryptIni(temp2, ini);
		System.out.println("encrypted ini");
		temp1.delete();
		temp2.delete();
	}


	/**
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	private void encryptIni(File source, File dest) throws FileNotFoundException, IOException
	{
		FileInputStream in = new FileInputStream(source);
		CryptOutputStream cout = new CryptOutputStream(new FileOutputStream(dest), 212);
		
		copyStream(in, cout);
	}


	/**
	 * @param in
	 * @param cout
	 * @throws IOException
	 */
	private void copyStream(InputStream in, OutputStream cout) throws IOException
	{
		try
		{
			byte[] buffer = new byte[16384];
			int len = -1;
			while ((len = in.read(buffer)) != -1)
			{	
				cout.write(buffer,0,len);
			}
		} 
		finally
		{
			try
			{
				cout.close();
			} 
			catch (Exception e)
			{
			}
	
			try
			{
				in.close();
			} 
			catch (Exception e)
			{
			}
		}
	}


	/**
	 * @param newServer
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	private void updateIni(File source, File dest, String newServer) throws FileNotFoundException, IOException
	{
		FileInputStream in = new FileInputStream(source);
		FileWriter out2 = new FileWriter(dest);
		try
		{
			LineNumberReader lnr = new LineNumberReader(new InputStreamReader(in));
			String line = null;
			while ( (line = lnr.readLine()) != null)
			{
				if (line.startsWith("ServerAddr="))
				{
					line = "ServerAddr=" + newServer;
				}
				
				out2.write(line+"\r\n");
			}
		} 
		finally
		{
			try
			{
				out2.close();
			} 
			catch (Exception e)
			{
			}

			try
			{
				in.close();
			} 
			catch (Exception e)
			{
			}
		}
	}


	/**
	 * @param source	encrypted file
	 * @param dest 		decrypted file
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	private void decryptIni(File source, File dest) throws FileNotFoundException, IOException
	{
		CryptInputStream cin = new CryptInputStream(new FileInputStream(source));
		FileOutputStream out = new FileOutputStream(dest);
		
		copyStream(cin, out);
	}
	
	private void createBackup(File source, File dest) throws FileNotFoundException, IOException
	{
		FileInputStream in = new FileInputStream(source);
		FileOutputStream out = new FileOutputStream(dest);
		
		copyStream(in, out);
	}
}
